<?php
/**
 * Description of Api_Tools_System
*/
class Api_Tools_System extends Api {

	public function __construct($queue, $websiteUrl) {
		parent::__construct($queue, $websiteUrl);
	}

	public static function getDomain() {
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl();
        return preg_replace('/https?:\/\//', '', $url);
    }

    public static function replaceSnippet($str, $needleStart, $needleEnd, $replacement) {
        $pos = strpos($str, $needleStart);
        $start = $pos === false ? 0 : $pos; //+ strlen($needleStart);
        $pos = strpos($str, $needleEnd, $start);
        $end = $start === false ? strlen($str) : $pos + strlen($needleEnd);
        return substr_replace($str, $replacement, $start, $end - $start);
    }

    public static function checkSnippetType($oldSnippet, $newSnippet){
        if(!empty(preg_match('/http:\/\/sa.seotoaster.com/', $oldSnippet)) || !empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $oldSnippet))){
            if (empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $oldSnippet))){
                $oldSnippet =  self::SEO_DATA_SAMBA_ANALYTIC_START . $oldSnippet . self::SEO_DATA_SAMBA_ANALYTIC_END;
            }
            if(!empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $newSnippet))){
                return self::replaceSnippet($oldSnippet, self::SEO_DATA_SAMBA_ANALYTIC_START, self::SEO_DATA_SAMBA_ANALYTIC_END,  $newSnippet);
            }
        }

        if (!empty(preg_match('/google-analytics.com\/ga.js/', $oldSnippet)) || !empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $oldSnippet))) {
            if (empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $oldSnippet))) {
                $oldSnippet = self::SEO_DATA_GOOGLE_ANALYTIC_START . $oldSnippet . self::SEO_DATA_GOOGLE_ANALYTIC_END;
            }
            if (!empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $newSnippet))) {
                return self::replaceSnippet($oldSnippet, self::SEO_DATA_GOOGLE_ANALYTIC_START, self::SEO_DATA_GOOGLE_ANALYTIC_END, $newSnippet);
            }
        }

        if (empty(preg_match('/' . self::SEO_DATA_NETCONTENT_START . '/', $oldSnippet)) &&
            empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $oldSnippet)) &&
            empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $oldSnippet))){
            $oldSnippet =  self::SEO_DATA_NETCONTENT_START . $oldSnippet . self::SEO_DATA_NETCONTENT_END;
        }
        if (!empty(preg_match('/' . self::SEO_DATA_NETCONTENT_START . '/', $oldSnippet)) && !empty(preg_match('/' .  self::SEO_DATA_NETCONTENT_START . '/', $newSnippet))) {
            return self::replaceSnippet($oldSnippet,  self::SEO_DATA_NETCONTENT_START, self::SEO_DATA_NETCONTENT_END, $newSnippet);
        }
        return $oldSnippet . $newSnippet;
    }

    public static function getRedirects($limit, $offset, $count = false) {
        $dbTable = new Zend_Db_Table();
        if(!$count) {
            $select = $dbTable->getAdapter()->select()->from('redirect', array(
                'id' => 'id',
                'pageId' => 'page_id',
                'fromUrl' => 'from_url',
                'toUrl' => 'to_url',
                'domainTo' => 'domain_to',
                'domainFrom' => 'domain_from'
            ))
                ->order(array('page_id', 'id'));
            if (!empty($offset)) {
                $select->limit($limit, $offset);
            } else {
                $select->limit($limit);
            }

            return $dbTable->getAdapter()->fetchAll($select);
        }

        return count(
            $dbTable->getAdapter()->fetchCol(
                $dbTable->getAdapter()->select()->from('redirect', array('count' => 'COUNT(id)'))->group('id')
            )
        );
    }

    public static function getUrls($limit, $offset, $count = false) {
        $dbTable = new Zend_Db_Table();
        $where = $dbTable->getAdapter()->quoteInto("`system` = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND draft = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND page_type NOT IN(?)", '4');

        if(!$count) {
            $select = $dbTable->getAdapter()->select()->from(array('p' => 'page'), null)
                ->joinLeft(array('o' => 'optimized'), 'p.id = o.page_id', null)
                ->columns(array(
                    'id' => 'p.id',
                    'url' => new Zend_Db_Expr('COALESCE(o.url, p.url)')
                ))->order(array('url'))
            ->where($where);

            if (!empty($offset)) {
                $select->limit($limit, $offset);
            } else {
                $select->limit($limit);
            }

            return $dbTable->getAdapter()->fetchAll($select);
        }

        return count(
            $dbTable->getAdapter()->fetchCol(
                $dbTable->getAdapter()->select()->from('page', array('count' => 'COUNT(id)'))->group('id')
                ->where($where)
            )
        );
    }

    public static function findUrls($query, $limit = 10) {
        $dbTable = new Zend_Db_Table();
        $where = $dbTable->getAdapter()->quoteInto("`system` = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND draft = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND page_type NOT IN(?)", '4');
        $where .= $dbTable->getAdapter()->quoteInto(" AND (COALESCE(o.url, p.url) LIKE ?)", $query . '%');

        $select = $dbTable->getAdapter()->select()->from(array('p' => 'page'), null)
            ->joinLeft(array('o' => 'optimized'), 'p.id = o.page_id', null)
            ->columns(array(
                'id' => 'p.id',
                'url' => new Zend_Db_Expr('COALESCE(o.url, p.url)')
            ))->order(array('url'))
            ->where($where)->limit($limit);

        return $dbTable->getAdapter()->fetchAll($select);
    }

    /**
     * First install remote registration flow
     *
     * @param string $token website token
     * @param string $websiteId website id (optional)
     * @return mixed
     * @throws Exceptions_SeotoasterPluginException
     */
    public static function firstWebsiteInstall($token, $websiteId = '')
    {
        $configMapper = Application_Model_Mappers_ConfigMapper::getInstance();
        $config = $configMapper->getConfig();
        $config['sambaToken'] = $token;
        $config['registered'] = '1';
        if (!empty($websiteId)) {
            $config['websiteId'] = $websiteId;
        }
        $configMapper->save($config);

        if (isset($config['analytics'])) {
            $analyticsType = $config['analytics'];
            if (isset($config['useGA'])) {
                $analyticsType = array($config['analytics'], $config['useGA']);
            }
        } else {
            $analyticsType = 'NA';
        }

        $config = array(
            'wicOrganizationName' => isset($config['wicOrganizationName']) ? $config['wicOrganizationName'] : '',
            'wicOrganizationDescription' => isset($config['wicOrganizationDescription']) ? $config['wicOrganizationDescription'] : '',
            'wicCorporateLogo' => isset($config['wicCorporateLogo']) ? $config['wicCorporateLogo'] : '',
            'wicOrganizationCountry' => isset($config['wicOrganizationCountry']) ? $config['wicOrganizationCountry'] : '',
            'wicAddress1' => isset($config['wicAddress1']) ? $config['wicAddress1'] : '',
            'wicAddress2' => isset($config['wicAddress2']) ? $config['wicAddress2'] : '',
            'wicCity' => isset($config['wicCity']) ? $config['wicCity'] : '',
            'wicCountryState' => isset($config['wicCountryState']) ? $config['wicCountryState'] : '',
            'wicZip' => isset($config['wicZip']) ? $config['wicZip'] : '',
            'wicLatitude' => isset($config['wicLatitude']) ? $config['wicLatitude'] : '',
            'wicLongitude' => isset($config['wicLongitude']) ? $config['wicLongitude'] : '',
            'wicPhone' => isset($config['wicPhone']) ? $config['wicPhone'] : '',
            'wicEmail' => isset($config['wicEmail']) ? $config['wicEmail'] : '',
            'wicFbAccount' => isset($config['wicFbAccount']) ? $config['wicFbAccount'] : '',
            'wicTwitAccount' => isset($config['wicTwitAccount']) ? $config['wicTwitAccount'] : '',
            'wicInstagramAccount' => isset($config['wicInstagramAccount']) ? $config['wicInstagramAccount'] : '',
            'wicLinkedInAccount' => isset($config['wicLinkedInAccount']) ? $config['wicLinkedInAccount'] : '',
            'wicYoutubeChannel' => isset($config['wicYoutubeChannel']) ? $config['wicYoutubeChannel'] : '',
            'wicRssChannel' => isset($config['wicRssChannel']) ? $config['wicRssChannel'] : '',
            'wicIndustryType' => isset($config['wicIndustryType']) ? $config['wicIndustryType'] : '',
            'wicPaymentType' => isset($config['wicPaymentType']) ? $config['wicPaymentType'] : '',
            'wicBusiness' => isset($config['wicBusiness']) ? $config['wicBusiness'] : '',
            'wicPayOnline' => isset($config['wicPayOnline']) ? $config['wicPayOnline'] : '',
            'wicAnalyticsType' => $analyticsType,
            'language' => isset($config['language']) ? $config['language'] : '',
            'sambaToken' => $token,
            'wicMSA' => isset($config['wicMSA']) ? $config['wicMSA'] : '',
            'wicReviewId' => isset($config['wicReviewId']) ? $config['wicReviewId'] : '',
            'wicVimeoChannel' => isset($config['wicVimeoChannel']) ? $config['wicVimeoChannel'] : '',
            'wicYelp' => isset($config['wicYelp']) ? $config['wicYelp'] : '',
            'wicGmbusiness' => isset($config['wicGmbusiness']) ? $config['wicGmbusiness'] : '',
            'wicSmsSenderId' => isset($config['wicSmsSenderId']) ? $config['wicSmsSenderId'] : ''
        );

        if (!empty($websiteId)) {
            $config['websiteId'] = $websiteId;
        }

        $newsFolderName = Newslog_Models_Mapper_ConfigurationMapper::getInstance()->fetchConfigParam('folder');
        if ($newsFolderName !== null) {
            $config['newsFolderName'] = $newsFolderName;
        }

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        $websitePath = $websiteHelper->getPath();

        $fileExtension = substr($config['wicCorporateLogo'], strrpos($config['wicCorporateLogo'], '.'));
        $logoFile = $websitePath . 'plugins/widcard/system/userdata/CorporateLogo' . $fileExtension;
        if (file_exists($logoFile)) {
            $imgBinary = fread(fopen($logoFile, "r"), filesize($logoFile));;
            $src = base64_encode($imgBinary);
            $config['wicCorporateLogo'] = array('extension' => substr($fileExtension, 1), 'src' => $src);
        } else {
            $config['wicCorporateLogo'] = '';
        }

        $config['websiteUrl'] = $websiteUrl;

        $widcardCustomParamsMapper = Widcard_Models_Mappers_WidcardCustomParamsMapper::getInstance();
        $customParams = $widcardCustomParamsMapper->getCustomParams();
        $config['wicCustomParams'] = $customParams;
        $config['wicSiteType'] = Widcard::TYPE_STANDARD;

        $register = Tools_Factory_PluginFactory::createPlugin('api', array(), array('websiteUrl' => $websiteUrl));
        $isEcommerce = $register->isShoppingInstalled();
        if (!empty($isEcommerce)){
            $config['wicSiteType'] = Widcard::TYPE_ECOMMERCE;
        }

        $registerResult = (object)$register::request('post', 'website', $config);//, truejson_decode();
        $data = array();
        if ($registerResult->done == false) {
            $data['done'] = false;
            $data['message'] = $registerResult->message;
        } else {
            $config = $configMapper->getConfig();
            if(property_exists($registerResult, 'websiteId')) {
                $config['websiteId'] = $registerResult->websiteId;
                $config['registered'] = true;
                $data['websiteId'] = $registerResult->websiteId;
            }
            $configMapper->save($config);
            $data['done'] = true;
        }

        return $data;

    }

}
