<?php

class Api_Leads_Leadmeetinglog extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('id');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead meeting log by id
     *
     * Resource:
     * : /api/leads/leadmeetinglog/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
    }

    /**
     * Create new lead task
     *
     * Resource:
     * : /api/leads/leadmeetinglog/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     * Update lead meeting log data
     *
     * Resource:
     * : /api/leads/leadmeetinglog/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead meeting log id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadMeetingLogId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadMeetingLogMapper = Leads_Mapper_LeadsMeetingLogMapper::getInstance();
            $leadMeetingMapper = Leads_Mapper_LeadsMeetingMapper::getInstance();
            $leadMeetingLogModel = $leadMeetingLogMapper->find($leadMeetingLogId);
            if (!$leadMeetingLogModel instanceof Leads_Model_LeadsMeetingLogModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Lead meeting log doesn\'t exists'));
            }

            $meetingModel = $leadMeetingMapper->find($leadMeetingLogModel->getMeetingId());

            if (!$meetingModel instanceof Leads_Model_LeadsMeetingModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Lead meeting doesn\'t exists'));
            }

            if (isset($data['type']) && $data['type'] === 'changeStatus') {
                if (empty($data['status'])) {
                    $meetingModel->setStatus(null);
                } elseif (!in_array($data['status'], Leads_Model_LeadsMeetingModel::$_allowedStatuses)) {
                    $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Meeting status not found'));
                } else {
                    $meetingModel->setStatus($data['status']);
                }

                $leadMeetingMapper->save($meetingModel);
                $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead meeting status has been changed'));
            } else {
                $leadMeetingLogModel->setOptions($data);
                $leadMeetingLogMapper->save($leadMeetingLogModel);
                $meetingModel->setAdditionalNotes($leadMeetingLogModel->getAdditionalContent());
            }

            $leadMeetingMapper->save($meetingModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead meeting log has been updated'));
        }
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));

    }

    /**
     * Delete meeting log record
     *
     * Resource:
     * : /api/leads/leadmeetinglog/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead meeting log id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
