<?php

class Api_Leads_Leadorganizecustomfieldsconfig extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    /**
     *
     * Resource:
     * : /api/leads/leadorganizecustomfieldsconfig/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $leadsOrganizeCustomFieldsMapper = Leads_Mapper_LeadsOrganizeCustomFieldsMapper::getInstance();
        $leadsCustomParamsConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
        $organizationsCustomParamsConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();

        $organizeCustomFieldsData = $leadsOrganizeCustomFieldsMapper->getDataConfig();
        $leadsCustomParams = $leadsCustomParamsConfigMapper->getCustomParamsPairs();
        $organizationsCustomParams = $organizationsCustomParamsConfigMapper->getCustomParamsPairs();

        $data = array(
            'leads' => array(),
            'organizations' => array()
        );

        //leads
        if(!empty($leadsCustomParams)) {
            if(empty($organizeCustomFieldsData['leads'])) {
                $draggableParamsIds = implode(',', array_keys($leadsCustomParams));
            } else {
                $draggableParamsIds = explode(',', $organizeCustomFieldsData['leads']);
                $draggableParamsIds = Tools_LeadTools::processDraggableCustomParamsIds($leadsCustomParams, $draggableParamsIds);
                $draggableParamsIds = implode(',', $draggableParamsIds);
            }

            $leadsCustomFieldsLabels = array();
            $draggableParamsIdsToView = explode(',', $draggableParamsIds);
            foreach ($draggableParamsIdsToView as $key => $id) {
                if(array_key_exists($id, $leadsCustomParams)) {
                    $leadsCustomFieldsLabels[] = array(
                        'id' => $id,
                        'label' => $leadsCustomParams[$id]
                    );
                }
            }

            if(!empty($leadsCustomFieldsLabels)) {
                $data['leads'] = $leadsCustomFieldsLabels;
            }

            $organizeCustomField = $leadsOrganizeCustomFieldsMapper->getByType('leads');

            if(!$organizeCustomField instanceof Leads_Model_LeadsOrganizeCustomFieldsModel) {
                $organizeCustomField = new Leads_Model_LeadsOrganizeCustomFieldsModel();
                $organizeCustomField->setType('leads');
            }

            $organizeCustomField->setDraggableParamsIds($draggableParamsIds);

            $leadsOrganizeCustomFieldsMapper->save($organizeCustomField);
        }

        //organizations
        if(!empty($organizationsCustomParams)) {
            if(empty($organizeCustomFieldsData['organizations'])) {
                $draggableParamsIds = implode(',', array_keys($organizationsCustomParams));
            } else {
                $draggableParamsIds = explode(',', $organizeCustomFieldsData['organizations']);
                $draggableParamsIds = Tools_LeadTools::processDraggableCustomParamsIds($organizationsCustomParams, $draggableParamsIds);
                $draggableParamsIds = implode(',', $draggableParamsIds);
            }

            $organizationsCustomFieldsLabels = array();
            $draggableParamsIdsToView = explode(',', $draggableParamsIds);
            foreach ($draggableParamsIdsToView as $key => $id) {
                if(array_key_exists($id, $organizationsCustomParams)) {
                    $organizationsCustomFieldsLabels[] = array(
                        'id' => $id,
                        'label' => $organizationsCustomParams[$id]
                    );
                }
            }

            if(!empty($organizationsCustomFieldsLabels)) {
                $data['organizations'] = $organizationsCustomFieldsLabels;
            }

            $organizeCustomField = $leadsOrganizeCustomFieldsMapper->getByType('organizations');

            if(!$organizeCustomField instanceof Leads_Model_LeadsOrganizeCustomFieldsModel) {
                $organizeCustomField = new Leads_Model_LeadsOrganizeCustomFieldsModel();
                $organizeCustomField->setType('organizations');
            }

            $organizeCustomField->setDraggableParamsIds($draggableParamsIds);

            $leadsOrganizeCustomFieldsMapper->save($organizeCustomField);
        }

        return array('organizeCustomFieldsData' => $data, 'status' => 'ok');
    }

    /**
     * Save custom tab
     *
     * * * Resource:
     * : /api/leads/leadorganizecustomfieldsconfig/
     *
     * @return array
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken,Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'), 'errorCode' => '1'), 200);
        }

        $type = filter_var($this->_request->getParam('type'), FILTER_SANITIZE_STRING);
        $draggableParamsIds = $this->_request->getParam('draggableParamsIds');

        if(!empty($type) && !empty($draggableParamsIds)) {
            $leadsOrganizeCustomFieldsMapper = Leads_Mapper_LeadsOrganizeCustomFieldsMapper::getInstance();
            $foundCustomConfig = $leadsOrganizeCustomFieldsMapper->getByType($type);

            $draggableParamsIds = implode(',', $draggableParamsIds);
            if($foundCustomConfig instanceof Leads_Model_LeadsOrganizeCustomFieldsModel) {
                $foundCustomConfig->setDraggableParamsIds($draggableParamsIds);
                $leadsOrganizeCustomFieldsMapper->save($foundCustomConfig);

                $this->_responseHelper->success(array('error' => '0'));
            }
        }
    }

    /**
     *
     * * * Resource:
     * : /api/leads/leadorganizecustomfieldsconfig/
     *
     *
     */
    public function putAction()
    {

    }

    /**
     *
     * * Resource:
     * : /api/leads/leadorganizecustomfieldsconfig/
     *
     * @return array
     */
    public function deleteAction()
    {

    }

}
