<?php

class Api_Leads_Leadtaskspreset extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/leads/leadtaskspreset/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $where = '';

        $leadsTasksPresetMapper = Leads_Mapper_LeadsTasksPresetMapper::getInstance();
        if (!empty($id)) {
            $where = $leadsTasksPresetMapper->getDbTable()->getAdapter()->quoteInto('pltp.id = ?', $id);
            $data = $leadsTasksPresetMapper->fetchAll($where, null, null, null, true, true);
        }  else {
            $data = $leadsTasksPresetMapper->fetchAll($where, null, $limit, $offset, false, false);
        }

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/leads/leadtaskspreset/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = $this->getRequest()->getParams();
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $data['presetName'] = filter_var($data['presetName'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
        $data['taskTitle'] = filter_var($data['taskTitle'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsTasksPresetMapper = Leads_Mapper_LeadsTasksPresetMapper::getInstance();

        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();

        $leadsTasksPresetModel = new Leads_Model_LeadsTasksPresetModel();
        $leadsTasksPresetModel->setCreatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
        $leadsTasksPresetModel->setUpdatedAt(null);
        $leadsTasksPresetModel->setPresetName($data['presetName']);
        $leadsTasksPresetModel->setCreatorId($userId);
        $leadsTasksPresetModel->setTaskTitle($data['taskTitle']);
        if (!empty($data['taskNote'])) {
            $leadsTasksPresetModel->setTaskNote(json_encode($data['taskNote']));
        } else {
            $leadsTasksPresetModel->setTaskNote('');
        }
        $leadsTasksPresetModel->setEditorId(null);
        $leadsTasksPresetMapper->save($leadsTasksPresetModel);

        return array(
            'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'preset_has_been_added')
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/leadtaskspreset/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }

            $leadsTasksPresetMapper = Leads_Mapper_LeadsTasksPresetMapper::getInstance();

            $leadsTasksPresetModel = $leadsTasksPresetMapper->find($data['id']);
            if ($leadsTasksPresetModel instanceof Leads_Model_LeadsTasksPresetModel) {
                $data['presetName'] = filter_var($data['presetName'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                $data['taskTitle'] = filter_var($data['taskTitle'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                if ($leadsTasksPresetModel->getPresetName() !== $data['presetName']) {
                    $presetWithNameExists = $leadsTasksPresetMapper->findByPresetName($data['presetName']);
                    if ($presetWithNameExists instanceof Leads_Model_LeadsTasksPresetModel) {
                        return array(
                            'error' => '1',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'preset_already_exists')
                        );
                    }
                }

                $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                $editorId = $session->getCurrentUser()->getId();

                $leadsTasksPresetModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadsTasksPresetModel->setPresetName($data['presetName']);
                $leadsTasksPresetModel->setTaskTitle($data['taskTitle']);
                if (!empty($data['taskNote'])) {
                    $leadsTasksPresetModel->setTaskNote($data['taskNote']);
                } else {
                    $leadsTasksPresetModel->setTaskNote('');
                }
                $leadsTasksPresetModel->setEditorId($editorId);
                $leadsTasksPresetMapper->save($leadsTasksPresetModel);

                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'preset_has_been_added')
                );

            }
        }

        return array(
            'error' => '1',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'preset_not_found')
        );

    }

    /**
     *
     * Resource:
     * : /api/leads/leadtaskspreset/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $leadsTasksPresetMapper = Leads_Mapper_LeadsTasksPresetMapper::getInstance();
        $leadsTasksPresetModel = $leadsTasksPresetMapper->find($id);
        if ($leadsTasksPresetModel instanceof Leads_Model_LeadsTasksPresetModel) {
            $leadsTasksPresetMapper->delete($id);
            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'preset_deleted')
            );
        } else {
            $this->_error();
        }

    }

}
