<?php

class Api_Leads_Leadwebhookurls extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    const PROCESS_WEBHOOKS_URLS_PATH = 'plugins/leads/system/cronscript/processWebhookNotifications.php';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/leads/leadwebhookurls/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $where = '';

        $leadsWebhookUrlsMapper = Leads_Mapper_LeadsWebhookUrlsMapper::getInstance();
        if (!empty($id)) {
            $where = $leadsWebhookUrlsMapper->getDbTable()->getAdapter()->quoteInto('plwu.id = ?', $id);
            $data = $leadsWebhookUrlsMapper->fetchAll($where, null, null, null, true, true);
        }  else {
            $data = $leadsWebhookUrlsMapper->fetchAll($where, null, $limit, $offset, false, false);
        }

        $data['cronPath'] = Zend_Registry::get('website')['path'].self::PROCESS_WEBHOOKS_URLS_PATH;

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/leads/leadwebhookurls/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = $this->getRequest()->getParams();
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $data['webhookName'] = filter_var($data['webhookName'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsWebhookUrlsMapper = Leads_Mapper_LeadsWebhookUrlsMapper::getInstance();

        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();

        $leadsWebhookUrlsModel = new Leads_Model_LeadsWebhookUrlsModel();

        $leadsWebhookUrlsModel->setWebhookName($data['webhookName']);
        $leadsWebhookUrlsModel->setWebhookType($data['webhookType']);
        $leadsWebhookUrlsModel->setWebhookActionType($data['webhookActionType']);
        $leadsWebhookUrlsModel->setActionUrl($data['actionUrl']);
        $leadsWebhookUrlsModel->setCreatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
        $leadsWebhookUrlsModel->setCreatorId($userId);
        $leadsWebhookUrlsModel->setEditorId(null);
        $leadsWebhookUrlsMapper->save($leadsWebhookUrlsModel);

        return array(
            'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Config has been added')
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/leadwebhookurls/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }

            $leadsWebhookUrlsMapper = Leads_Mapper_LeadsWebhookUrlsMapper::getInstance();

            $leadsWebhookUrlsModel = $leadsWebhookUrlsMapper->find($data['id']);
            if ($leadsWebhookUrlsModel instanceof Leads_Model_LeadsWebhookUrlsModel) {
                $data['webhookName'] = filter_var($data['webhookName'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);

                if ($leadsWebhookUrlsModel->getWebhookName() !== $data['webhookName']) {
                    $presetWithNameExists = $leadsWebhookUrlsMapper->findByConfigName($data['webhookName']);
                    if ($presetWithNameExists instanceof Leads_Model_LeadsWebhookUrlsModel) {
                        return array(
                            'error' => '1',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'Config already exists')
                        );
                    }
                }

                $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                $editorId = $session->getCurrentUser()->getId();

                $leadsWebhookUrlsModel->setWebhookName($data['webhookName']);
                $leadsWebhookUrlsModel->setWebhookType($data['webhookType']);
                $leadsWebhookUrlsModel->setWebhookActionType($data['webhookActionType']);
                $leadsWebhookUrlsModel->setActionUrl($data['actionUrl']);
                $leadsWebhookUrlsModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadsWebhookUrlsModel->setEditorId($editorId);
                $leadsWebhookUrlsMapper->save($leadsWebhookUrlsModel);

                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Config has been added')
                );

            }
        }

        return array(
            'error' => '1',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Config not found')
        );

    }

    /**
     *
     * Resource:
     * : /api/leads/leadwebhookurls/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $leadsWebhookUrlsMapper = Leads_Mapper_LeadsWebhookUrlsMapper::getInstance();
        $leadsWebhookUrlsModel = $leadsWebhookUrlsMapper->find($id);
        if ($leadsWebhookUrlsModel instanceof Leads_Model_LeadsWebhookUrlsModel) {
            $leadsWebhookUrlsMapper->delete($id);
            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Config deleted')
            );
        } else {
            $this->_error();
        }

    }

}
