<?php

class Api_Leadsdashboard_Changetasks extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * Special prefix for unique tranlation
     */
    const TRANSLATION_PREFIX = 'leads_';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    protected $_loggedUserRoleId = null;

    /**
     * User restricted lifecycle id
     *
     * @var null
     */
    protected $_restrictedLifeCycleId = 0;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();
        $this->_restrictedLifeCycleId = Tools_LeadTools::getLifecycleIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/changetasks/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $filterDateFrom = filter_var($this->_request->getParam('filterDateFrom'), FILTER_SANITIZE_STRING);

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }
        $data = array();

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     * : /api/leadsdashboard/changetasks/
     *
     * @return JSON
     */
    public function postAction()
    {
        $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, Api_Leads_Leads::LEADS_SECURE_TOKEN);
        $specialCloseType = filter_var($this->_request->getParam('specialCloseType'), FILTER_SANITIZE_STRING);

        if (!$tokenValid) {
            $this->_responseHelper->fail('Invalid token');
        }

        $translator = Zend_Registry::get('Zend_Translate');

        $taskIds = array_filter(filter_var_array($this->_request->getParam('tasksIds', false), FILTER_SANITIZE_STRING));
        $additionalComment = filter_var($this->_request->getParam('additionalComment', false), FILTER_SANITIZE_STRING);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $applyFullFilter = filter_var($this->_request->getParam('applyFullFilter'), FILTER_SANITIZE_NUMBER_INT);

        if (!empty($this->_request->getParam('filter')) && is_array($this->_request->getParam('filter'))) {
            $filterData = filter_var_array($this->_request->getParam('filter'), FILTER_SANITIZE_STRING);
        } else {
            $filterData = array();
        }


        $dateTodayEnd = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('+1 day')) . ' 00:00:00');

        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        $count = 2;
        if (empty($taskIds) && empty($applyFullFilter) && empty($specialCloseType)) {
            $this->_responseHelper->fail(array('error' => '1', 'message' => $translator->translate(self::TRANSLATION_PREFIX . 'Please_specify_atleast_one_task'), 'count' => $count));
        }


        $closedByUserId = $this->_sessionHelper->getCurrentUser()->getId();

        $where = null;
        $filter = array();
        $sortOrder = 'plt.due_date DESC';

        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        if (empty($applyFullFilter)) {
            $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.id IN (?)', $taskIds);

            if ($specialCloseType === Leads_Model_LeadsTaskModel::LEAD_TASK_COMPLETE_TYPE_ALL_TASKS_FOR_SELECTED_LEADS) {
                $leadIds = $leadsTaskMapper->getLeadsByTaskIds($taskIds);
                $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.lead_id IN (?)', $leadIds);
                $where .= ' AND ' . $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');
            }

            if ($specialCloseType === Leads_Model_LeadsTaskModel::LEAD_TASK_COMPLETE_TYPE_ALL_WITH_IDENTICAL_TITLE) {
                $taskTitles = $leadsTaskMapper->getTaskTitlesByTaskIds($taskIds);
                $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.title IN (?)', $taskTitles);
                $where .= ' AND ' . $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');
            }

        } else {
            $userId = $filterData['userId'];
            $subtype = $filterData['querySubType'];

            $showAllUsers = false;
            if ($userId == '-1') {
                $showAllUsers = true;
            }

            if ($showAllUsers === false) {
                if (!empty($userId) && empty($restrictedAccess)) {
                    $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.user_id = ?', $userId);
                } else {
                    $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.user_id = ?',
                        $currentUserId);
                }
            }

            if (!empty($where)) {
                $where .= ' AND ';
            }

            $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');

            if ($subtype === Tools_LeadTools::TIMELINE_TYPE_UPCOMING) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.due_date > ?', $dateTodayEnd);
            } elseif ($subtype === Api_Leads_Leads::FILTER_TASK_OVERDUE) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_overdue = ?', '1');
            } else {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.due_date < ?', $dateTodayEnd);
            }
        }

        if (empty($applyFullFilter)) {
            $data = $leadsTaskMapper->fetchAllData($where, $sortOrder, $limit, null);
        } else {
            $data = $leadsTaskMapper->fetchAllData($where, $sortOrder, $limit, $offset);
        }

        $totalRecords = $data['totalRecords'];
        if (empty($data['data'])) {
            $this->_responseHelper->fail(array('error' => '1', 'message' => $translator->translate('Tasks have been completed'), 'count' => $totalRecords));
        }

        if (!empty($offset)) {
            $count = $limit + $offset;
        }

        foreach ($data['data'] as $taskInfo) {
            $result = Tools_LeadTasksTools::changeSimpleTaskStatus($taskInfo['taskId'], $closedByUserId, $additionalComment);
            if (!empty($result['error'])) {
                $this->_responseHelper->fail(array('error' => '1', 'message' => $result['message'], 'count' => $count));
            }
        }

        if ($limit >= $totalRecords) {
            $this->_responseHelper->fail(array('error' => '1', 'message' => $translator->translate('Tasks have been completed'), 'count' => $count));
        }

        $this->_responseHelper->success(array('error' => '0', 'count' => $count));
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
