<?php

class Api_Leadsdashboard_Counttaskstocomplete extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * Special prefix for unique tranlation
     */
    const TRANSLATION_PREFIX = 'leads_';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    protected $_loggedUserRoleId = null;

    /**
     * User restricted lifecycle id
     *
     * @var null
     */
    protected $_restrictedLifeCycleId = 0;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();
        $this->_restrictedLifeCycleId = Tools_LeadTools::getLifecycleIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/counttaskstocomplete/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $taskIds = filter_var_array($this->_request->getParam('tasksIds'), FILTER_SANITIZE_NUMBER_INT);
        $specialCloseType = filter_var($this->_request->getParam('specialCloseType'), FILTER_SANITIZE_STRING);

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        if ($specialCloseType === Leads_Model_LeadsTaskModel::LEAD_TASK_COMPLETE_TYPE_ALL_TASKS_FOR_SELECTED_LEADS) {
            $leadIds = $leadsTaskMapper->getLeadsByTaskIds($taskIds);
            $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.lead_id IN (?)', $leadIds);
            $where .= ' AND ' . $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');
        }

        if ($specialCloseType === Leads_Model_LeadsTaskModel::LEAD_TASK_COMPLETE_TYPE_ALL_WITH_IDENTICAL_TITLE) {
            $taskTitles = $leadsTaskMapper->getTaskTitlesByTaskIds($taskIds);
            $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.title IN (?)', $taskTitles);
            $where .= ' AND ' . $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');
        }

        $result = $leadsTaskMapper->fetchAllData($where, null, null, null, false, true);
        if (empty($result)) {
            $data['totalRecords'] = 0;
        } else {
            $data['totalRecords'] = $result['totalRecords'];
        }

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     * : /api/leadsdashboard/changetasks/
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
