<?php

class Api_Leadsdashboard_Manageusers extends Api_Service_Abstract
{

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     *
     * Resource:
     * : /api/leadsdashboard/manageusers/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $filter = $this->_request->getParam('filter');
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $where = null;
        $order = 'u.id DESC';

        $userMapper = Application_Model_Mappers_UserMapper::getInstance();

        if (!empty($filter)) {
            if (!empty($filter['byRoleId'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $userMapper->getDbTable()->getAdapter()->quoteInto('u.role_id = ?', $filter['byRoleId']);
            } else {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $userMapper->getDbTable()->getAdapter()->quoteInto('u.role_id IN (?)', array(Tools_Security_Acl::ROLE_ADMIN, Tools_Security_Acl::ROLE_SUPERADMIN, Shopping::ROLE_SALESPERSON));
            }


            if (!empty($filter['byText'])) {
                $filterByName = filter_var($filter['byText'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                $attributeValues = explode(' ', trim($filterByName));
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= ' (' . '(';
                foreach ($attributeValues as $key => $attrVal) {
                    $where .= $userMapper->getDbTable()->getAdapter()->quoteInto('u.full_name LIKE ?',
                        '%' . $attrVal . '%');

                    if (count($attributeValues) > $key + 1) {
                        $where .= ' AND ';
                    }
                }

                $where .= ')';

                $where .= ' OR ' . $userMapper->getDbTable()->getAdapter()->quoteInto('u.email LIKE ?',
                        '%' . trim($filterByName) . '%');

                $where .= ')';
            }
        } else {
            if (!empty($where)) {
                $where .= ' AND ';
            }
            $where .= $userMapper->getDbTable()->getAdapter()->quoteInto('u.role_id IN (?)', array(Tools_Security_Acl::ROLE_ADMIN, Tools_Security_Acl::ROLE_SUPERADMIN, Shopping::ROLE_SALESPERSON));
        }

        if (!empty($id)) {
            if (!empty($where)) {
                $where .= ' AND ';
            }

            $where .= $userMapper->getDbTable()->getAdapter()->quoteInto('u.id = ?', $id);

            $data = $userMapper->fetchData($where, $order, $limit, $offset, true, true);

            if (!empty($data) && $data['role_id'] === Tools_Security_Acl::ROLE_SUPERADMIN) {
                $data = array();
            }

            $timezoneAbbr = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $data['timezone'], 'T');
            $data['timezoneAbbr'] = $timezoneAbbr;
        } else {
            $data = $userMapper->fetchData($where, $order, $limit, $offset);
        }

        $currency = Zend_Registry::get('Zend_Currency');
        $currencySymbol = preg_replace('~[\w]~', '', $currency->getSymbol());
        $acl = Zend_Registry::get('acl');
        $roles = array_filter($acl->getRoles(), function ($role) {
            return (($role === Tools_Security_Acl::ROLE_ADMIN) || $role === Shopping::ROLE_SALESPERSON);
        });

        $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
        $mobileMasks = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_MOBILE);
        $desktopMasks = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);
        $phoneCountryCodes = Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);
        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultTimezone = $configHelper->getConfig('userDefaultTimezone');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');

        $timezones = DateTimeZone::listIdentifiers(DateTimeZone::ALL);
        array_pop($timezones);

        $attributes = $userMapper->fetchUniqueAttributesNames();
        $userCustomAttributes = array();
        if (!empty($id)) {
            $userModel = $userMapper->find($id);
            if ($userModel instanceof Application_Model_Models_User) {
                $userModelWithAttributes = $userMapper->loadUserAttributes($userModel);
                $userCustomAttributesRes = $userModelWithAttributes->getAttributes();
                if (!empty($userCustomAttributesRes)) {
                    foreach ($userCustomAttributesRes as $key => $val) {
                        array_push($userCustomAttributes, array('name' => $key, 'value' => $val));
                    }
                }
            }
        }

        $data['userCustomAttributes'] = $userCustomAttributes;
        $data['customParamsList'] = $attributes;

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $activateMfa = $configHelper->getConfig('activateMfa');

        $data['additionalInfo'] = array(
            'currencySymbol' => $currencySymbol,
            'userDate' => Tools_System_Tools::convertDateFromTimezone('now'),
            'rolesListSearch' => array(
                Tools_Security_Acl::ROLE_SUPERADMIN => array('label' => Tools_Security_Acl::ROLE_SUPERADMIN),
                Tools_Security_Acl::ROLE_ADMIN => array('label' => Tools_Security_Acl::ROLE_ADMIN),
                Shopping::ROLE_SALESPERSON => array('label' => Shopping::ROLE_SALESPERSON),
            ),
            'rolesCreation' => $roles,
            'mobileMasks' => $mobileMasks,
            'desktopMasks' => $desktopMasks,
            'phoneCountryCodes' => $phoneCountryCodes,
            'userDefaultTimezone' => $userDefaultTimezone,
            'userDefaultMobileCountryCode' => $userDefaultMobileCountryCode,
            'timezonesList' => $timezones,
            'activateMfa' => $activateMfa

        );

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, Leads::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
        }

        $fullName = trim(filter_var($this->_request->getParam('fullName'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $desktopNumber = trim(filter_var($this->_request->getParam('desktopNumber'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $desktopCountryCode = trim(filter_var($this->_request->getParam('userDesktopCountryCode'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $mobileNumber = trim(filter_var($this->_request->getParam('mobileNumber'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $mobileCountryCode = trim(filter_var($this->_request->getParam('userMobileCountryCode'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $email = trim(filter_var($this->_request->getParam('email'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $sendInvite = trim(filter_var($this->_request->getParam('sendInvite'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $voipPhone = trim(filter_var($this->_request->getParam('voipPhone'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $roleId = trim(filter_var($this->_request->getParam('userRole'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $password = trim(filter_var($this->_request->getParam('userPassword'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $timezone = trim(filter_var($this->_request->getParam('userTimezone'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $avatarLink = trim(filter_var($this->_request->getParam('avatarLink'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $personalCalendarUrl = trim(filter_var($this->_request->getParam('personalCalendarUrl'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $userPrefix = trim(filter_var($this->_request->getParam('userPrefix'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $signature = trim($this->_request->getParam('signature'));
        $subscribed = trim(filter_var($this->_request->getParam('userSubscribed'), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));

        $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
        $currentLoggedUserRoleId = $currentUser->getRoleId();
        if ($currentLoggedUserRoleId !== Tools_Security_Acl::ROLE_SUPERADMIN && $currentLoggedUserRoleId !== Tools_Security_Acl::ROLE_ADMIN) {
            return array(
                'error' => '1',
                'message' => 'Access not allowed'
            );
        }

        if (empty($fullName)) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'please_specify_full_name')
            );
        }

        if (empty($email)) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'please_specify_email')
            );
        }

        if (empty($password)) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'please_specify_password')
            );
        }

        if (empty($roleId)) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'please_specify_role')
            );
        }

        $mobilePhone = Tools_System_Tools::cleanNumber($mobileNumber);
        $desktopPhone = Tools_System_Tools::cleanNumber($desktopNumber);
        if (!empty($mobileCountryCode)) {
            $mobileCountryPhoneCode = Zend_Locale::getTranslation($mobileCountryCode, 'phoneToTerritory');
            $mobileCountryCodeValue = '+' . $mobileCountryPhoneCode;
        } else {
            $mobileCountryCodeValue = null;
        }
        if (!empty($desktopCountryCode)) {
            $mobileCountryPhoneCode = Zend_Locale::getTranslation($desktopCountryCode, 'phoneToTerritory');
            $desktopCountryCodeValue = '+' . $mobileCountryPhoneCode;
        } else {
            $desktopCountryCodeValue = null;
        }

        if ($roleId === Tools_Security_Acl::ROLE_SUPERADMIN) {
            return array(
                'error' => '1',
                'message' => $translator->translate('not allowed')
            );
        }

        $currentLoggedUserRole = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();
        if ($roleId === Tools_Security_Acl::ROLE_ADMIN && ($currentLoggedUserRole !== Tools_Security_Acl::ROLE_ADMIN && $currentLoggedUserRole !== Tools_Security_Acl::ROLE_SUPERADMIN)) {
            return array(
                'error' => '1',
                'message' => $translator->translate('not allowed')
            );
        }

        $userMapper = Application_Model_Mappers_UserMapper::getInstance();

        $userModel = $userMapper->findByEmail($email);
        if ($userModel instanceof Application_Model_Models_User) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'user_with_this_email_already_exists')
            );
        }

        if (Tools_System_Tools::isEmailValid($email) === false) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'specify_valid_email_address')
            );
        }

        $passwordValidator = new Zend_Validate_StringLength(array(
            'encoding' => 'UTF-8',
            'min' => 4
        ));

        if ($passwordValidator->isValid($password) === false) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'specify_valid_password')
            );
        }

        if ($roleId === Tools_Security_Acl::ROLE_SUPERADMIN) {
            return array(
                'error' => '1',
                'message' => 'Creating superadmin role not allowed'
            );
        }


        $user = new Application_Model_Models_User();
        $user->setRoleId($roleId);
        $user->setDesktopCountryCode($desktopCountryCode);
        $user->setDesktopCountryCodeValue($desktopCountryCodeValue);
        $user->setDesktopPhone($desktopPhone);
        $user->setMobileCountryCode($mobileCountryCode);
        $user->setMobileCountryCodeValue($mobileCountryCodeValue);
        $user->setMobilePhone($mobilePhone);
        $user->setTimezone($timezone);
        $user->setSignature($signature);
        $user->setVoipPhone($voipPhone);
        $user->setFullName($fullName);
        $user->setPrefix($userPrefix);
        $user->setPassword($password);
        $user->setSubscribed($subscribed);
        $user->setAvatarLink($avatarLink);
        $user->setEmail($email);
        $user->setPersonalCalendarUrl($personalCalendarUrl);

        $uId = $userMapper->save($user);

        if (!empty($attrNamesArr) && !empty($attrValuesArr)) {
            foreach ($attrNamesArr as $key => $value) {
                if (empty($value)) {
                    continue;
                }
                $user->setAttribute($value, $attrValuesArr[$key]);
            }
            if (empty($currentUserId)) {
                $user->setId((int)$uId);
            }
            $userMapper->saveUserAttributes($user);
        }

        $createUserInfoStatus = Tools_System_Tools::firePluginMethodByTagName(
            'usercreate', 'createUserInfo',
            array(
                'userId' => $uId,
                'email' => $email
            )
        );

        if (!empty($sendInvite)) {
            $user->removeAllObservers();
            $resetToken = Tools_System_Tools::saveResetToken($email, $uId);
            if ($resetToken instanceof Application_Model_Models_PasswordRecoveryToken) {
                $user->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_Mail_SystemMailWatchdog::TRIGGER_USERINVITATION,
                    'resetToken' => $resetToken
                )));

                $user->notifyObservers();

                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'invitation_email_has_been_sent')
                );
            } else {
                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'cant_generate_reset_token')
                );
            }
        }


        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'user_has_been_created')
        );


    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['userId']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {

            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, Leads::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
            }

            $userId = filter_var($data['userId'], FILTER_SANITIZE_NUMBER_INT);

            $fullName = trim(filter_var($data['fullName'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $desktopNumber = trim(filter_var($data['desktopNumber'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $desktopCountryCode = trim(filter_var($data['userDesktopCountryCode'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $mobileNumber = trim(filter_var($data['mobileNumber'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $mobileCountryCode = trim(filter_var($data['userMobileCountryCode'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $email = trim(filter_var($data['email'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $sendInvite = trim(filter_var($data['sendInvite'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $voipPhone = trim(filter_var($data['voipPhone'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $roleId = trim(filter_var($data['userRole'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $password = trim(filter_var($data['userPassword'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $timezone = trim(filter_var($data['userTimezone'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $avatarLink = trim(filter_var($data['avatarLink'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $personalCalendarUrl = trim(filter_var($data['personalCalendarUrl'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $userPrefix = trim(filter_var($data['userPrefix'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $signature = trim($data['signature']);
            $subscribed = trim(filter_var($data['userSubscribed'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $enabledReports = filter_var($data['enabledReports'], FILTER_SANITIZE_NUMBER_INT);
            $enabledExcludeWeekends = filter_var($data['enabledExcludeWeekends'], FILTER_SANITIZE_NUMBER_INT);
            $reportCcEmail = trim(filter_var($data['reportCcEmail'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
            $preferableTime = filter_var($data['preferableTime'], FILTER_SANITIZE_NUMBER_INT);
            $reportTypesList = filter_var_array($data['reportTypes'], FILTER_SANITIZE_STRING);
            $mfaEnabled = trim(filter_var($data['mfaEnabled'], FILTER_SANITIZE_NUMBER_INT));

            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $currentLoggedUserRoleId = $currentUser->getRoleId();
            if ($currentLoggedUserRoleId !== Tools_Security_Acl::ROLE_SUPERADMIN && $currentLoggedUserRoleId !== Tools_Security_Acl::ROLE_ADMIN) {
                return array(
                    'error' => '1',
                    'message' => 'Access not allowed'
                );
            }

            if (empty($fullName)) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'please_specify_full_name')
                );
            }

            if (empty($email)) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'please_specify_email')
                );
            }

            if (empty($roleId)) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'please_specify_role')
                );
            }

            if ($roleId === Tools_Security_Acl::ROLE_SUPERADMIN) {
                return array(
                    'error' => '1',
                    'message' => 'Creating superadmin role not allowed'
                );
            }

            if (Tools_System_Tools::isEmailValid($email) === false) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'specify_valid_email_address')
                );
            }

            if (!empty($enabledReports)) {
                if (!empty($reportCcEmail)) {
                    $receiveReportsCcEmails = explode(',', $reportCcEmail);
                    foreach ($receiveReportsCcEmails as $receiveReportCCEmail) {
                        if (Tools_System_Tools::isEmailValid($receiveReportCCEmail) === false) {
                            return array(
                                'error' => '1',
                                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'specify_valid_report_cc_email_address')
                            );
                        }
                    }

                }
            }

            $userMapper = Application_Model_Mappers_UserMapper::getInstance();

            $userModel = $userMapper->find($userId);
            if (!$userModel instanceof Application_Model_Models_User) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'user_not_found')
                );
            }

            if (!empty($password)) {
                $passwordValidator = new Zend_Validate_StringLength(array(
                    'encoding' => 'UTF-8',
                    'min' => 4
                ));

                if ($passwordValidator->isValid($password) === false) {
                    return array(
                        'error' => '1',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'specify_valid_password')
                    );
                }
            }

            $mobilePhone = Tools_System_Tools::cleanNumber($mobileNumber);
            $desktopPhone = Tools_System_Tools::cleanNumber($desktopNumber);
            if (!empty($mobileCountryCode)) {
                $mobileCountryPhoneCode = Zend_Locale::getTranslation($mobileCountryCode, 'phoneToTerritory');
                $mobileCountryCodeValue = '+' . $mobileCountryPhoneCode;
            } else {
                $mobileCountryCodeValue = null;
            }
            if (!empty($desktopCountryCode)) {
                $mobileCountryPhoneCode = Zend_Locale::getTranslation($desktopCountryCode, 'phoneToTerritory');
                $desktopCountryCodeValue = '+' . $mobileCountryPhoneCode;
            } else {
                $desktopCountryCodeValue = null;
            }

            $oldUserEmailAddress = $userModel->getEmail();

            if ($oldUserEmailAddress !== $email) {
                $where = $userMapper->getDbTable()->getAdapter()->quoteInto('u.email = ?', $email);
                $where .= ' AND ' . $userMapper->getDbTable()->getAdapter()->quoteInto('u.id <> ?', $userId);
                $anotherUserExists = $userMapper->fetchData($where, null, null, null, true, true);
                if (!empty($anotherUserExists)) {
                    return array(
                        'error' => '1',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'user_with_this_email_already_exists')
                    );
                }
            }

            $userCustomAttributes = $data['userCustomAttributes'];
            $listOfCustomAttributes = $data['listOfCustomAttributes'];

            $userModel->setRoleId($roleId);
            $userModel->setDesktopCountryCode($desktopCountryCode);
            $userModel->setDesktopCountryCodeValue($desktopCountryCodeValue);
            $userModel->setDesktopPhone($desktopPhone);
            $userModel->setMobileCountryCode($mobileCountryCode);
            $userModel->setMobileCountryCodeValue($mobileCountryCodeValue);
            $userModel->setMobilePhone($mobilePhone);
            $userModel->setTimezone($timezone);
            $userModel->setSignature($signature);
            $userModel->setVoipPhone($voipPhone);
            $userModel->setFullName($fullName);
            $userModel->setPrefix($userPrefix);
            if (empty($password)) {
                $userModel->setPassword(null);
            } else {
                $userModel->setPassword($password);
            }

            $userModel->setSubscribed($subscribed);
            $userModel->setAvatarLink($avatarLink);
            $userModel->setEmail($email);
            $userModel->setPersonalCalendarUrl($personalCalendarUrl);

            if (!empty($enabledReports)) {
                $userModel->setReceiveReports(1);
            } else {
                $userModel->setReceiveReports(0);
            }

            if (!empty($mfaEnabled)) {
                $userModel->setEnabledMfa(1);
            } else {
                $userModel->setEnabledMfa(0);
            }

            $userModel->setExcludeWeekends($enabledExcludeWeekends);
            $userModel->setReceiveReportsCcEmail($reportCcEmail);
            $userModel->setReceiveReportsPreferableTime($preferableTime);

            $userModel->setReceiveReportsTypesList(implode(',', $reportTypesList));

            $uId = $userModel->getId();


            if (!empty($userCustomAttributes)) {
                foreach ($userCustomAttributes as $userCustomAttribute) {
                    if (empty($userCustomAttribute['value'])) {
                        continue;
                    }

                    $customAttrValue = filter_var($userCustomAttribute['value'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                    $customOfCustomAttrName = preg_replace('~[^-\w\d]~', '', $userCustomAttribute['name']);

                    $userModel->setAttribute($customOfCustomAttrName, $customAttrValue);
                }
                if (empty($currentUserId)) {
                    $userModel->setId((int)$uId);
                }
            }

            if (!empty($listOfCustomAttributes)) {
                foreach ($listOfCustomAttributes as $listOfCustomAttribute) {
                    if (empty($listOfCustomAttribute['value'])) {
                        continue;
                    }

                    $listOfCustomAttrValue = filter_var($listOfCustomAttribute['value'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                    $listOfCustomAttrName = preg_replace('~[^-\w\d]~', '', $listOfCustomAttribute['name']);

                    $userModel->setAttribute($listOfCustomAttrName, $listOfCustomAttrValue);
                }

                if (empty($currentUserId)) {
                    $userModel->setId((int)$uId);
                }
            }

            if (!empty($userCustomAttributes) || !empty($listOfCustomAttributes)) {
                Application_Model_Mappers_UserMapper::getInstance()->saveUserAttributes($userModel);
            } else {
                $dbTable = new Zend_Db_Table('user_attributes');

                $dbTable->delete(array('user_id = ?' => $userId));
            }

            $userMapper->save($userModel);


            if ($oldUserEmailAddress !== $email && !empty($userId)) {
                $updateUserInfoStatus = Tools_System_Tools::firePluginMethodByTagName(
                    'userupdate', 'updateUserInfo',
                    array(
                        'userId' => $userId,
                        'oldEmail' => $oldUserEmailAddress,
                        'newEmail' => $email
                    )
                );
            }

            if (!empty($sendInvite)) {
                $userModel->removeAllObservers();
                $resetToken = Tools_System_Tools::saveResetToken($email, $userModel->getId());
                if ($resetToken instanceof Application_Model_Models_PasswordRecoveryToken) {
                    $userModel->registerObserver(new Tools_Mail_Watchdog(array(
                        'trigger' => Tools_Mail_SystemMailWatchdog::TRIGGER_USERINVITATION,
                        'resetToken' => $resetToken
                    )));

                    $userModel->notifyObservers();

                    return array(
                        'error' => '0',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'invitation_email_has_been_sent')
                    );
                } else {
                    return array(
                        'error' => '0',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'cant_generate_reset_token')
                    );
                }
            }

            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'user_has_been_updated')
            );

        }

        return array(
            'error' => '1',
            'message' => $translator->translate('missing user id')
        );

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            return array('error' => '1', 'message' => 'User id is missing');
        }

        $translator = Zend_Registry::get('Zend_Translate');
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();

        try {
            $userModel = $userMapper->find($id);
            if ($userModel instanceof Application_Model_Models_User) {
                $userMapper->delete($userModel);
                $userDeleteExternalStatus = Tools_System_Tools::firePluginMethodByTagName('userdelete', 'deleteSystemUser', array('userId' => $id, 'email' => $userModel->getEmail()));
                return array('error' => '0', 'message' => $translator->translate('User has been deleted'));
                exit;
            } else {
                return array('error' => '1', 'message' => $translator->translate('Can\'t remove user...'));
            }

        } catch (Exception $exception) {
            $userDeleteErrorMessage = Tools_System_Tools::firePluginMethod('userdeleteerror', 'systemUserDeleteErrorMessage');

            if (!empty($userDeleteErrorMessage)) {
                return array('error' => '1', 'message' => $userDeleteErrorMessage);
            }

            return array('error' => '1', 'message' => $translator->translate('Can\'t remove user...'));
        }
    }

}
