<?php

class Api_Leadsdashboard_Manageusersgeneralinfo extends Api_Service_Abstract
{


    /**
     * Service ERP secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     *
     * Resource:
     * : /api/erpmembersdash/usersgeneralinfo/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $currency = Zend_Registry::get('Zend_Currency');
        $currencySymbol = preg_replace('~[\w]~', '', $currency->getSymbol());

        $data = array();

        $acl = Zend_Registry::get('acl');
        $roles = array_filter($acl->getRoles(), function($role){
            return (($role === Tools_Security_Acl::ROLE_ADMIN) || $role === Shopping::ROLE_SALESPERSON);
        });

        $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
        $mobileMasks = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_MOBILE);
        $desktopMasks = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);
        $phoneCountryCodes =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);
        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultTimezone = $configHelper->getConfig('userDefaultTimezone');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');

        $timezones = DateTimeZone::listIdentifiers(DateTimeZone::ALL);
        array_pop($timezones);

        $data['additionalInfo'] = array(
            'currencySymbol' => $currencySymbol,
            'userDate' => Tools_System_Tools::convertDateFromTimezone('now'),
            'rolesListSearch' => array(
                Tools_Security_Acl::ROLE_SUPERADMIN => array('label' => Tools_Security_Acl::ROLE_SUPERADMIN),
                Tools_Security_Acl::ROLE_ADMIN => array('label' => Tools_Security_Acl::ROLE_ADMIN),
                Shopping::ROLE_SALESPERSON => array('label' => Shopping::ROLE_SALESPERSON),
            ),
            'rolesCreation' => $roles,
            'mobileMasks' => $mobileMasks,
            'desktopMasks' => $desktopMasks,
            'phoneCountryCodes' => $phoneCountryCodes,
            'userDefaultTimezone' => $userDefaultTimezone,
            'userDefaultMobileCountryCode' => $userDefaultMobileCountryCode,
            'timezonesList' => $timezones,
        );

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
