<?php

/**
 * LeadsEmailReportLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsEmailReportLogMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsEmailReportLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsEmailReportLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsEmailReportLogDbTable';

    /**
     * Save purchase log model to DB
     * @param $model Leads_Model_LeadsEmailReportLogModel
     * @return Leads_Model_LeadsEmailReportLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'user_id' => $model->getUserId(),
            'sent_at_date' => $model->getSentAtDate(),
            'processed_at' => $model->getProcessedAt(),
            'status' => $model->getStatus()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get tasks stats data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset with timezone offset
     * @param array $additionalParamsForSelect select query params
     */
    public function fetchTasksStatsData($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false, $singleRecord = false, $withTimezoneOffset = true, $additionalParamsForSelect = array())
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        if ($withTimezoneOffset === true) {
            $actionRecordedDate = new Zend_Db_Expr("CONVERT_TZ(`plal`.`created_at`, '+00:00', '$mysqlOffset')");
            $taskStartDate = new Zend_Db_Expr("CONVERT_TZ(`plta`.`expired_at`, '+00:00', '$mysqlOffset')");
            $taskDueDate = new Zend_Db_Expr("CONVERT_TZ(`plta`.`due_date`, '+00:00', '$mysqlOffset')");
        } else {
            $actionRecordedDate = 'plal.created_at';
            $taskStartDate = 'plta.expired_at';
            $taskDueDate = 'plta.due_date';
        }

        $params = array(
            'pld.id',
            'pld.lead_first_name',
            'pld.lead_last_name',
            'pld.lead_email',
            'plta.title',
            'actionRecorderDate' => $actionRecordedDate,
            'plta.is_task_completed',
            'plta.task_completed_by',
            'plta.is_task_overdue',
            'plta.notes',
            'taskStartDate' => $taskStartDate,
            'taskDueDate' => $taskDueDate,

        );

        $params = array_merge($additionalParamsForSelect, $params);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'),
                $params
            )
            ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
            ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array())
            ->joinLeft(array('plta' => 'plugin_leads_tasks'), 'plta.lead_id=pld.id', array())
            ->joinLeft(array('pltl' => 'plugin_leads_tasks_log'), 'pltl.action_log_id=plal.id', array());

        if (!empty($having)) {
            $select->having($having);
        }

        $select->group('plta.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plta.id))'));
            $count = array_merge($count, $additionalParamsForSelect);

            $select->from(array('pld' => 'plugin_leads_data'), $count)
                ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
                ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array())
                ->joinLeft(array('plta' => 'plugin_leads_tasks'), 'plta.lead_id=pld.id', array())
                ->joinLeft(array('pltl' => 'plugin_leads_tasks_log'), 'pltl.action_log_id=plal.id', array());

            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }

    }

    /**
     * Get tasks stats data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset with timezone offset
     * @param array $additionalParamsForSelect select query params
     */
    public function fetchMeetingsStatsData($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false, $singleRecord = false, $withTimezoneOffset = true, $additionalParamsForSelect = array())
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        if ($withTimezoneOffset === true) {
            $meetingStartDate = new Zend_Db_Expr("CONVERT_TZ(`plm`.`start_time_date`, '+00:00', '$mysqlOffset')");
        } else {
            $meetingStartDate = 'plm.start_time_date';
        }

        $params = array(
            'pld.id',
            'pld.lead_first_name',
            'pld.lead_last_name',
            'pld.lead_email',
            'plm.subject',
            'meetingStartDate' => $meetingStartDate
        );

        $params = array_merge($additionalParamsForSelect, $params);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'),
                $params
            )
            ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
            ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array())
            ->joinLeft(array('plml' => 'plugin_leads_meeting_log'), 'plml.action_log_id=plal.id', array())
            ->joinLeft(array('plm' => 'plugin_leads_meeting'), 'plml.meeting_id=plm.id', array());


        if (!empty($having)) {
            $select->having($having);
        }

        $select->group('plm.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plm.id))'));
            $count = array_merge($count, $additionalParamsForSelect);

            $select->from(array('pld' => 'plugin_leads_data'), $count)
                ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
                ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array())
                ->joinLeft(array('plm' => 'plugin_leads_meeting'), 'plm.lead_id=pld.id', array())
                ->joinLeft(array('plml' => 'plugin_leads_meeting_log'), 'plml.action_log_id=plal.id', array());

            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }

    }

    /**
     * Get notes stats data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset with timezone offset
     * @param array $additionalParamsForSelect select query params
     */
    public function fetchNotesStatsData($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false, $singleRecord = false, $withTimezoneOffset = true, $additionalParamsForSelect = array())
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        if ($withTimezoneOffset === true) {
            $dateCreated = new Zend_Db_Expr("CONVERT_TZ(`pln`.`date_created`, '+00:00', '$mysqlOffset')");
        } else {
            $dateCreated = 'pln.date_created';
        }

        $params = array(
            'pld.id',
            'pld.lead_first_name',
            'pld.lead_last_name',
            'pld.lead_email',
            'pln.note',
            'dateCreated' => $dateCreated
        );

        $params = array_merge($additionalParamsForSelect, $params);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'),
                $params
            )
            ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
            ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array())
            ->joinLeft(array('pln' => 'plugin_leads_notes'), 'pln.lead_id=pld.id', array());

        if (!empty($having)) {
            $select->having($having);
        }

        $select->group('pln.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(pln.id))'));
            $count = array_merge($count, $additionalParamsForSelect);

            $select->from(array('pld' => 'plugin_leads_data'), $count)
                ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
                ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array())
                ->joinLeft(array('pln' => 'plugin_leads_notes'), 'pln.lead_id=pld.id', array());

            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }

    }

    /**
     * Find by user sent at date
     *
     * @param int $userId user id
     * @param string $sentAtDate sent at date
     * @return Leads_Model_LeadsEmailReportLogModel|null
     */
    public function findByUserSentAtDate($userId, $sentAtDate)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('sent_at_date = ?', $sentAtDate);
        return $this->_findWhere($where);
    }

    /**
     * Delete log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_email_report_log', $where);

    }

}
