<?php

/**
 * LeadsOrganizeCustomFieldsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOrganizeCustomFieldsMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOrganizeCustomFieldsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOrganizeCustomFieldsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOrganizeCustomFieldsDbTable';

    /**
     * Save custom fields order model to DB
     * @param $model Leads_Model_LeadsOrganizeCustomFieldsModel
     * @return Leads_Model_LeadsOrganizeCustomFieldsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'type' => $model->getType(),
            'draggable_params_ids' => $model->getDraggableParamsIds(),
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get organize custom fields config by type
     *
     * @param string $type type
     * @return null
     */
    public function getByType($type)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('type = ?', $type);

        return $this->_findWhere($where);
    }

    /**
     * Get all notes pairs
     *
     * @return array
     */
    public function getDataConfig()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_organize_custom_fields_config', array('type', 'draggable_params_ids'));

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }


    /**
     * Delete lead note record
     *
     * @param int $id note id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_organize_custom_fields_config', $where);

    }

}
