<?php

/**
 * LeadsValidateEmailsQueueMapper.php
 *
 *
 * @method Leads_Mapper_LeadsValidateEmailsQueueMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsValidateEmailsQueueMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsValidateEmailsQueueQueueModel';

    protected $_dbTable = 'Leads_DbTable_LeadsValidateEmailsQueueDbTable';

    /**
     * Save lead task model to DB
     * @param $model Leads_Model_LeadsValidateEmailsQueueQueueModel
     * @return Leads_Model_LeadsValidateEmailsQueueQueueModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'creator_id' => $model->getCreatorId(),
            'created_at' => $model->getCreatedAt(),
            'process_at' => $model->getProcessAt(),
            'processed_at' => $model->getProcessedAt(),
            'delayed_to' => $model->getDelayedTo(),
            'lead_ids' => $model->getLeadIds(),
            'status' => $model->getStatus(),
            'processed_leads' => $model->getProcessedLeads(),
            'not_processed_leads' => $model->getNotProcessedLeads(),
            'balance' => $model->getBalance(),
            'error_info' => $model->getErrorInfo(),
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     *
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param string $having mysql having
     * @param bool $withTimezoneOffset with timezone offset
     *
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $having = '',
        $withTimezoneOffset = true
    )
    {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        if ($withTimezoneOffset === true) {
            $createdDate = new Zend_Db_Expr("CONVERT_TZ(`plveq`.`created_at`, '+00:00', '$mysqlOffset')");
            $processAtDate = new Zend_Db_Expr("CONVERT_TZ(`plveq`.`process_at`, '+00:00', '$mysqlOffset')");
        } else {
            $createdDate = 'plveq.created_at';
            $processAtDate = 'plveq.process_at';
        }

        $params = array(
            'plveq.id',
            'plveq.lead_ids',
            'plveq.delayed_to',
            'plveq.processed_at',
            'plveq.status',
            'plveq.processed_leads',
            'plveq.not_processed_leads',
            'plveq.balance',
            'plveq.error_info',
            'taskCreatedAt' => $createdDate,
            'processAtDate' => $processAtDate
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plveq' => 'plugin_leads_validate_emails_queue'),
                $params
            );

        if (!empty($having)) {
            $select->having($having);
        }

        $select->group('plveq.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plveq.id))'));

            $select->from(array('plveq' => 'plugin_leads_validate_emails_queue'), $count);

            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            if (empty($count['count'])) {
                $count['count'] = 0;
            }

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }


    /**
     * Update queue status
     *
     * @param int $queueId queue id
     * @param string $status queue status
     * @throws Zend_Db_Adapter_Exception
     */
    public function updateQueueStatus($queueId, $status)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $queueId);
        $data = array('status' => $status);
        $this->getDbTable()->getAdapter()->update('plugin_leads_validate_emails_queue', $data, $where);

    }

    /**
     * Delete lead record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_validate_emails_queue', $where);

    }

}
