<?php

/**
 * LeadsWebhookNotificationsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsWebhookNotificationsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsWebhookNotificationsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsWebhookNotificationsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsWebhookNotificationsDbTable';

    /**
     * Save lead webhook notifications
     * @param $model Leads_Model_LeadsWebhookNotificationsModel
     * @return Leads_Model_LeadsWebhookNotificationsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'status'                  => $model->getStatus(),
            'webhook_type'            => $model->getWebhookType(),
            'resource_id'             => $model->getResourceId(),
            'webhook_action_type'     => $model->getWebhookActionType(),
            'additional_info'         => $model->getAdditionalInfo(),
            'created_at'              => $model->getCreatedAt(),
            'updated_at'              => $model->getUpdatedAt(),
            'sent_notifications_data' => $model->getSentNotificationsData(),
            'sent_at'                 => $model->getSentAt(),
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    public function findScheduledLog($id, $resource, $webhookActionType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('status = ?', 'scheduled');
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('resource_id = ?', $id);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('webhook_type = ?', $resource);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('webhook_action_type = ?', $webhookActionType);

        return $this->_findWhere($where);
    }

    public function getNotSendNotifications($webhookType, $webhookActionType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('status = ?', 'scheduled');
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('webhook_type = ?', $webhookType);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('webhook_action_type = ?', $webhookActionType);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plwn' => 'plugin_leads_webhook_notifications'),
                array(
                    'plwn.*'
                )
            );

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * @param int $id record id
     * @param string $status notification status
     */
    public function changeStatus($id, $status)
    {
        $data = array('status' => $status);
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $this->getDbTable()->update($data, $where);
    }

    /**
     * @param array $ids
     * @param string $status notification status
     */
    public function changeStatuses($ids, $status)
    {
        $data = array('status' => $status);
        $where = $this->getDbTable()->getAdapter()->quoteInto('id IN (?)', $ids);
        $this->getDbTable()->update($data, $where);
    }

    /**
     * Delete record
     *
     * @param int $id id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_webhook_notifications', $where);

    }

}
