<?php

/**
 * Class Tools_LeadEmailReportsTools
 */
class Tools_LeadEmailReportsTools
{

    const REPORT_TYPE_TASKS = 'tasks';

    const REPORT_TYPE_MEETINGS = 'meetings';

    const REPORT_TYPE_NOTES = 'notes';

    const TASKS_COMPLETED = 'completed';

    const TASKS_CREATED = 'created';

    const TASKS_EXPIRES_TODAY = 'expires_today';

    const MEETING_CREATED = 'created';

    const MEETING_COMPLETED= 'completed';

    const MEETING_TODAY = 'today';

    /**
     *
     * Allowed tasks subtypes
     *
     * @var string[]
     *
     */
    public static $_allowedTasksSubtypes = array(
        self::TASKS_COMPLETED => self::TASKS_COMPLETED,
        self::TASKS_CREATED => self::TASKS_CREATED,
        self::TASKS_EXPIRES_TODAY => self::TASKS_EXPIRES_TODAY,
    );

    /**
     *
     * Allowed meetings subtypes
     *
     *
     * @var string[]
     */
    public static $_allowedMeetingSubtypes = array(
        self::MEETING_CREATED => self::MEETING_CREATED,
        self::MEETING_COMPLETED => self::MEETING_COMPLETED,
        self::MEETING_TODAY => self::MEETING_TODAY,
    );

    /**
     * Get tasks reports data
     *
     * @param int $userId user id
     * @param string $userTimezone user timezone
     * @param string $type report type
     * @return array
     */
    public static function getTasksReportsData($userId, $userTimezone, $type = self::TASKS_COMPLETED)
    {
        $data = array();

        $previousPeriod = '-1 day';

        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $userModel = $userMapper->find($userId);
        if ($userModel instanceof Application_Model_Models_User) {
            $excludeWeekends = $userModel->getExcludeWeekends();
            $currentDayOfTheWeekNumber = (int)Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $userTimezone, 'N');
            if (!empty($excludeWeekends)) {
                if ($currentDayOfTheWeekNumber === 1) {
                    $previousPeriod = '-3 day';
                }
            }
        }

        if ($type === self::TASKS_COMPLETED) {
            $startWithDate = Tools_System_Tools::convertDateFromTimezone($previousPeriod, 'UTC', $userTimezone, 'Y-m-d');

            $startDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 00:00:00',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');
            $endDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 23:59:59',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');

            $leadsEmailReportLogMapper = Leads_Mapper_LeadsEmailReportLogMapper::getInstance();

            $where = $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '1');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?', 'task_completed');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $startDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $endDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.user_id = ?', $userId);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('pltl.is_task_completed = ?', '1');
            $data = $leadsEmailReportLogMapper->fetchTasksStatsData($where, null, null, null, false, false, false);
        }


        if ($type === self::TASKS_CREATED) {
            $startWithDate = Tools_System_Tools::convertDateFromTimezone($previousPeriod, 'UTC', $userTimezone, 'Y-m-d');

            $startDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 00:00:00',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');
            $endDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 23:59:59',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');

            $leadsEmailReportLogMapper = Leads_Mapper_LeadsEmailReportLogMapper::getInstance();

            $where = $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '0');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?', 'task');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('pltl.is_task_completed = ?', '0');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.lead_type = ?', 'outbound_task');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.event_type = ?', 'expired');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $startDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $endDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.user_id = ?', $userId);
            $data = $leadsEmailReportLogMapper->fetchTasksStatsData($where, null, null, null, false, false, false);
        }

        if ($type === self::TASKS_EXPIRES_TODAY) {
            $startWithDate = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $userTimezone, 'Y-m-d');

            $startDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 00:00:00',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');
            $endDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 23:59:59',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');

            $leadsEmailReportLogMapper = Leads_Mapper_LeadsEmailReportLogMapper::getInstance();

            $where = $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '0');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.due_date > ?', $startDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.due_date < ?', $endDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plta.user_id = ?', $userId);
            $data = $leadsEmailReportLogMapper->fetchTasksStatsData($where, null, null, null, false, false, false);
        }


        return $data;

    }

    /**
     * Get meeting reports data
     *
     * @param int $userId user id
     * @param string $userTimezone user timezone
     * @param string $type report type
     * @return array
     */
    public static function getMeetingReportsData($userId, $userTimezone, $type = self::MEETING_CREATED)
    {
        $data = array();

        $previousPeriod = '-1 day';

        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $userModel = $userMapper->find($userId);
        if ($userModel instanceof Application_Model_Models_User) {
            $excludeWeekends = $userModel->getExcludeWeekends();
            $currentDayOfTheWeekNumber = (int)Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $userTimezone, 'N');
            if (!empty($excludeWeekends)) {
                if ($currentDayOfTheWeekNumber === 1) {
                    $previousPeriod = '-3 day';
                }
            }
        }

        if ($type === self::MEETING_CREATED) {
            $startWithDate = Tools_System_Tools::convertDateFromTimezone($previousPeriod, 'UTC', $userTimezone, 'Y-m-d');

            $startDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 00:00:00',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');
            $endDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 23:59:59',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');

            $leadsEmailReportLogMapper = Leads_Mapper_LeadsEmailReportLogMapper::getInstance();

            $where = $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $startDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $endDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.creator_id = ?', $userId);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?', 'meeting');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.event_type  = ?', 'expired');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plal.lead_type  = ?', 'outbound_meeting');
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plml.meeting_status = ?', 'created');
            $data = $leadsEmailReportLogMapper->fetchMeetingsStatsData($where, null, null, null, false, false, false);
        }

        if ($type === self::MEETING_COMPLETED) {
            $startWithDate = Tools_System_Tools::convertDateFromTimezone($previousPeriod, 'UTC', $userTimezone, 'Y-m-d');

            $startDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 00:00:00',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');
            $endDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 23:59:59',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');

            $leadsEmailReportLogMapper = Leads_Mapper_LeadsEmailReportLogMapper::getInstance();

            $where = $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.start_time_date > ?', $startDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.start_time_date < ?', $endDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.creator_id = ?', $userId);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.status = ?', Leads_Model_LeadsMeetingModel::LEAD_MEETING_STATUS_COMPLETED);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plml.meeting_status = ?', 'upcoming');
            $data = $leadsEmailReportLogMapper->fetchMeetingsStatsData($where, null, null, null, false, false, false);
        }

        if ($type === self::MEETING_TODAY) {
            $startWithDate = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $userTimezone, 'Y-m-d');

            $startDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 00:00:00',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');
            $endDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 23:59:59',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');

            $leadsEmailReportLogMapper = Leads_Mapper_LeadsEmailReportLogMapper::getInstance();

            $where = $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.start_time_date > ?', $startDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.start_time_date < ?', $endDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('plm.creator_id = ?', $userId);
            $data = $leadsEmailReportLogMapper->fetchMeetingsStatsData($where, null, null, null, false, false, false);
        }

        return $data;
    }

    /**
     * Get notes reports data
     *
     * @param int $userId user id
     * @param string $userTimezone user timezone
     * @param string $type report type
     * @return array
     */
    public static function getNotesReportsData($userId, $userTimezone, $type = self::REPORT_TYPE_NOTES)
    {
        $data = array();

        $previousPeriod = '-1 day';

        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $userModel = $userMapper->find($userId);
        if ($userModel instanceof Application_Model_Models_User) {
            $excludeWeekends = $userModel->getExcludeWeekends();
            $currentDayOfTheWeekNumber = (int)Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $userTimezone, 'N');
            if (!empty($excludeWeekends)) {
                if ($currentDayOfTheWeekNumber === 1) {
                    $previousPeriod = '-3 day';
                }
            }
        }

        if ($type === self::REPORT_TYPE_NOTES) {
            $startWithDate = Tools_System_Tools::convertDateFromTimezone($previousPeriod, 'UTC', $userTimezone, 'Y-m-d');

            $startDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 00:00:00',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');
            $endDate = Tools_System_Tools::convertDateFromTimezone($startWithDate . ' 23:59:59',
                $userTimezone, 'UTC', 'Y-m-d H:i:s');

            $leadsEmailReportLogMapper = Leads_Mapper_LeadsEmailReportLogMapper::getInstance();

            $where = $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('pln.date_created > ?', $startDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('pln.date_created < ?', $endDate);
            $where .= ' AND ' . $leadsEmailReportLogMapper->getDbTable()->getAdapter()->quoteInto('pln.user_id = ?', $userId);
            $data = $leadsEmailReportLogMapper->fetchNotesStatsData($where, null, null, null, false, false, false);
        }

        return $data;
    }


}
