<?php

/**
 * Class Tools_LeadSmsTools
 */
class Tools_LeadSmsTools
{

    public static $_acceptedFileTypesMms = array(
        'jpg,jpeg,png,bmp,gif,mp3,avi,mpeg,mp4,webm,mov'
    );

    public static $_acceptedFilesMimeTypesMms = array(
        'image/png',
        'image/jpeg',
        'image/jpg',
        'image/gif',
        'image/bmp',
        'audio/mpeg3',
        'audio/mpeg',
        'video/avi',
        'video/x-msvideo',
        'video/mp4',
        'video/mpeg',
        'video/quicktime'
    );

    public static $_acceptedFileTypesMmsArray = array(
        'jpg',
        'jpeg',
        'png',
        'bmp',
        'gif',
        'mp3',
        'avi',
        'mpeg',
        'mp4',
        'webm',
        'mov'
    );

    /**
     * Parse widgets lexems
     *
     * @param $smsMessage
     * @param $leadData
     * @param $websiteUrl
     * @param $excludeTags
     * @return string
     */
    public static function addDictionarySmsFields($smsMessage, $leadData, $websiteUrl, $excludeTags = '') {
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $leadOwnerId = $leadData->getOwnerId();

        $leadOwnerFullname = '';
        $leadOwnerEmail = '';
        $leadOwnerFullMobilePhone = '';
        $leadOwnerFullDesktopPhone = '';
        $leadOwnerSignature = '';
        $leadOrganizationName = '';
        $leadOwnerPersonalcalendarurl = '';
        $leadOwnerAvatarlink = '';
        $leadOrganizationWebsite = '';
        $fulldesktopphone = '';
        $fullmobilephone = '';


        if(!empty($leadOwnerId)) {
            $ownerModel = $userMapper->find($leadOwnerId);

            if ($ownerModel instanceof Application_Model_Models_User) {
                $leadOwnerFullname = $ownerModel->getFullName();
                $leadOwnerEmail = $ownerModel->getEmail();
                $leadOwnerFullMobilePhone = $ownerModel->getMobileCountryCodeValue() . $ownerModel->getMobilePhone();
                $leadOwnerFullDesktopPhone = $ownerModel->getDesktopCountryCodeValue() . $ownerModel->getDesktopPhone();
                $leadOwnerSignature = $ownerModel->getSignature();
                $leadOwnerPersonalcalendarurl = $ownerModel->getPersonalCalendarUrl();
                $leadOwnerAvatarlink = $ownerModel->getAvatarLink();
            }
        }
        $organizationId = $leadData->getOrganizationId();
        if(!empty($organizationId)) {
            $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
            $leadOrganization = $leadOrganizationMapper->find($organizationId);

            if($leadOrganization instanceof Leads_Model_LeadsOrganizationModel) {
                $leadOrganizationName = $leadOrganization->getOrganizationName();
                $leadOrganizationWebsite = $leadOrganization->getOrganizationWebsite();
            }
        }

        $fullDesktopPhone = $leadData->getLeadPhone();
        if (!empty($fullDesktopPhone)) {
            $fulldesktopphone = $leadData->getLeadPhoneCountryCodeValue() . $fullDesktopPhone;
        }

        $fullMobilePhone = $leadData->getLeadMobile();
        if (!empty($fullMobilePhone)) {
            $fullmobilephone = $leadData->getLeadMobileCountryCodeValue() . $fullMobilePhone;
        }

        $prefix = $leadData->getPrefix();
        if (empty($prefix)) {
            $prefix = '';
        }

        $position = $leadData->getLeadPosition();
        if (empty($position)) {
            $position = '';
        }

        $middlename = $leadData->getMiddleName();
        if (empty($middlename)) {
            $middlename = '';
        }

        $dictionary = array(
            '$lead:firstname'                => $leadData->getLeadFirstName(),
            '$lead:lastname'                 => $leadData->getLeadLastName(),
            '$lead:source'                   => $leadData->getLeadSource(),
            '$leadowner:fullname'            => $leadOwnerFullname,
            '$leadowner:email'               => $leadOwnerEmail,
            '$leadowner:fullmobilephone'     => $leadOwnerFullMobilePhone,
            '$leadowner:fulldesktopphone'    => $leadOwnerFullDesktopPhone,
            '$leadowner:signature'           => $leadOwnerSignature,
            '$lead:organizationname'         => $leadOrganizationName,
            '$website:url'                   => $websiteUrl,
            '$lead:organizationlogo'         => '',
            '$lead:organizationlogo:src'     => '',
            '$leadowner:personalcalendarurl' => $leadOwnerPersonalcalendarurl,
            '$leadowner:avatarlink'          => '',
            '$leadowner:avatarlink:src'      => $leadOwnerAvatarlink,
            '$lead:leadfullnamelink'         => $leadData->getLeadFirstName() . ' ' .$leadData->getLeadLastName(),
            '$lead:organizationnamelink'     => $leadOrganizationName,
            '$lead:organizationwebsite'      => $leadOrganizationWebsite,
            '$lead:fulldesktopphone'         => $fulldesktopphone,
            '$lead:fullmobilephone'          => $fullmobilephone,
            '$lead:prefix'                   => $prefix,
            '$lead:position'                 => $position,
            '$lead:middlename'               => $middlename,

        );

        $entityParser = new Tools_Content_EntityParser();

        $entityParser->addToDictionary($dictionary);

        if (!empty($excludeTags)) {
            return $entityParser->parse(strip_tags($smsMessage, $excludeTags));
        }

        return $entityParser->parse(strip_tags($smsMessage));
    }

    /**
     * Get path to file
     *
     * @param string $file file
     * @return string
     */
    public static function generateStoredMmsName($file)
    {
        $storedData = array();
        $fileExtension = pathinfo($file['name'], PATHINFO_EXTENSION);
        $fileName = pathinfo($file['name'], PATHINFO_FILENAME);

        $storedData['fileExtension'] = $fileExtension;
        $storedData['fileName'] = $fileName;
        $storedData['fileStoredName'] = Tools_LeadDocumentsTools::generateUniqueFileStoredName($fileName, $fileExtension);
        $storedData['fileHash'] = Tools_LeadDocumentsTools::generateUniqueHash($fileName);

        return $storedData;
    }

    /**
     * Get path to file
     *
     * @param string $fileStoredName file stored name
     * @return string
     */
    public static function getFilePath($fileStoredName, $dirName)
    {
        $mmsPath = self::getMmsDirectoryPath($dirName);

        if(!empty($mmsPath)) {
            if(!is_dir($mmsPath)) {
                try {
                    Tools_Filesystem_Tools::mkDir($mmsPath);
                } catch (Exceptions_SeotoasterException $e) {
                    error_log($e->getMessage());
                }
            }

            return $mmsPath . DIRECTORY_SEPARATOR . $fileStoredName;
        }

        return '';
    }

    /**
     * Get lead MMS directory path
     *
     * @param $dirName
     * @return string
     */
    public static function getMmsDirectoryPath($dirName, $getUrl = false)
    {
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');

        if(!empty($dirName)) {
            if($getUrl) {
                $pathUrl = $websiteHelper->getUrl();
            } else {
                $pathUrl = $websiteHelper->getPath();
            }

            $mmsPath = $pathUrl . 'plugins' . DIRECTORY_SEPARATOR . 'leads' .
                DIRECTORY_SEPARATOR . 'web' . DIRECTORY_SEPARATOR . Tools_LeadDocumentsTools::FILES_FOLDER
                . DIRECTORY_SEPARATOR . 'lead-mms-media-files' . DIRECTORY_SEPARATOR . $dirName;

            return $mmsPath;
        }

        return '';
    }

    /**
     * Delete MMS media files by leadId
     *
     * @param $leadId
     * @return void
     * @throws Exceptions_SeotoasterException
     */
    public static function deleteMmsMediaFiles($leadId)
    {
        $leadMmsMediaFilesPath = self::getMmsDirectoryPath($leadId);

        if(!empty($leadMmsMediaFilesPath) && is_dir($leadMmsMediaFilesPath)) {
            $folderContent = Tools_Filesystem_Tools::scanDirectory($leadMmsMediaFilesPath, false, true);
            if(!empty($folderContent)) {
                try {
                    Tools_Filesystem_Tools::deleteDir($leadMmsMediaFilesPath);
                } catch (Exception $e) {
                    //$e->getMessage();
                }
            }
        }
    }

    public static function copyMadiaFiles($fromLeadId, $toLeadId) {
        if(!empty($fromLeadId) && !empty($toLeadId)) {
            $leadMmsMediaFilesPath = self::getMmsDirectoryPath($fromLeadId);

            if(!empty($leadMmsMediaFilesPath) && is_dir($leadMmsMediaFilesPath)) {
                $folderContent = Tools_Filesystem_Tools::scanDirectory($leadMmsMediaFilesPath, false, true);

                if(!empty($folderContent)) {
                    foreach ($folderContent as $mediaFile) {
                        $savePathToCopy = self::getFilePath($mediaFile, $toLeadId);

                        if(!empty($savePathToCopy)) {
                            $savePath = $leadMmsMediaFilesPath . DIRECTORY_SEPARATOR . $mediaFile;
                            Tools_Filesystem_Tools::copy($savePath, $savePathToCopy);
                        }
                    }
                }
            }
        }

        return true;
    }
}
