<?php

/**
 * Class Tools_LeadWebhookNotificationsTools
 */
class Tools_LeadWebhookNotificationsTools
{

    /**
     * Allowed webhoook notification cart statuses
     *
     * @var array
     */
    public static $_allowedCartStatuses = array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_PARTIAL);


    public static function addToWebhookNotificationsLog($model, $resource, $webhookActionType)
    {
        if ($model instanceof Leads_Model_LeadsModel) {
            $leadsWebhookNotificationsMapper = Leads_Mapper_LeadsWebhookNotificationsMapper::getInstance();

            $leadsWebhookNotificationsModel = null;
            if($webhookActionType == 'modification') {
                $leadsWebhookNotificationsModel = $leadsWebhookNotificationsMapper->findScheduledLog($model->getId(), $resource, $webhookActionType);
            }

            if(!$leadsWebhookNotificationsModel instanceof Leads_Model_LeadsWebhookNotificationsModel) {
                $leadsWebhookNotificationsModel = new Leads_Model_LeadsWebhookNotificationsModel();

                $leadsWebhookNotificationsModel->setStatus(Leads_Model_LeadsWebhookNotificationsModel::STATUS_SCHEDULEDED);
                $leadsWebhookNotificationsModel->setWebhookType($resource);
                $leadsWebhookNotificationsModel->setResourceId($model->getId());
                $leadsWebhookNotificationsModel->setWebhookActionType($webhookActionType);
                $leadsWebhookNotificationsModel->setCreatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
            } else {
                $leadsWebhookNotificationsModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
            }
            $leadsWebhookNotificationsModel->setAdditionalInfo('');

            $leadsWebhookNotificationsMapper->save($leadsWebhookNotificationsModel);
        } elseif ($model instanceof Leads_Model_LeadsOrganizationModel) {
            //@todo implement processing for organization
        }
    }

    /**
     * Add order to the webhook notification flow
     *
     * @param int $orderId order id
     */
    public static function addOrderToWebhook($orderId)
    {
        $cartSessionMapper = Models_Mapper_CartSessionMapper::getInstance();
        $cartSessionModel = $cartSessionMapper->find($orderId);
        if ($cartSessionModel instanceof Models_Model_CartSession) {
            $cartStatus = $cartSessionModel->getStatus();
            if (!in_array($cartStatus, self::$_allowedCartStatuses)) {
                return array();
            }

            $additionalInfo = '';

            $webHookType = Leads_Model_LeadsWebhookNotificationsModel::WEBHOOK_TYPE_ORDER;

            $webHookActionType = Leads_Model_LeadsWebhookNotificationsModel::WEBHOOK_ACTION_TYPE_ORDER_COMPLETED;
            if ($cartStatus === Models_Model_CartSession::CART_STATUS_COMPLETED) {
                $webHookActionType = Leads_Model_LeadsWebhookNotificationsModel::WEBHOOK_ACTION_TYPE_ORDER_COMPLETED;
            }

            if ($cartStatus === Models_Model_CartSession::CART_STATUS_PARTIAL) {
                $webHookActionType = Leads_Model_LeadsWebhookNotificationsModel::WEBHOOK_ACTION_TYPE_ORDER_PARTIAL;
                $additionalInfo = json_encode($cartSessionModel->toArray());
            }

            $resourceId = $orderId;
            $leadsWebhookNotificationsMapper = Leads_Mapper_LeadsWebhookNotificationsMapper::getInstance();
            $leadsWebhookNotificationsModel = $leadsWebhookNotificationsMapper->findScheduledLog($resourceId, $webHookType, $webHookActionType);
            if (!$leadsWebhookNotificationsModel instanceof Leads_Model_LeadsWebhookNotificationsModel) {
                $leadsWebhookNotificationsModel = new Leads_Model_LeadsWebhookNotificationsModel();

                $leadsWebhookNotificationsModel->setStatus(Leads_Model_LeadsWebhookNotificationsModel::STATUS_SCHEDULEDED);
                $leadsWebhookNotificationsModel->setWebhookType($webHookType);
                $leadsWebhookNotificationsModel->setResourceId($resourceId);
                $leadsWebhookNotificationsModel->setWebhookActionType($webHookActionType);
                $leadsWebhookNotificationsModel->setCreatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadsWebhookNotificationsModel->setAdditionalInfo($additionalInfo);

                $leadsWebhookNotificationsMapper->save($leadsWebhookNotificationsModel);
            }
        }
    }

    /**
     * Notify webhook endpoints
     *
     * @param string $webhookType webhook type
     * @param string $webhookActionType webhook action type
     */
    public static function notifyWebhookEndpoints($webhookType, $webhookActionType)
    {
        $leadsWebhookUrlsMapper = Leads_Mapper_LeadsWebhookUrlsMapper::getInstance();
        $webhookUrlsConfigs = $leadsWebhookUrlsMapper->getPresetsData($webhookType, $webhookActionType);

        if (!empty($webhookUrlsConfigs) && is_array($webhookUrlsConfigs)) {
            $leadsWebhookNotificationsMapper = Leads_Mapper_LeadsWebhookNotificationsMapper::getInstance();

            foreach ($webhookUrlsConfigs as $webhookConfig) {
                $notificationData = $leadsWebhookNotificationsMapper->getNotSendNotifications($webhookConfig['webhook_type'], $webhookConfig['webhook_action_type']);

                if (!empty($notificationData)) {
                    $webhookType = $webhookConfig['webhook_type'];
                    $notificationUrl = $webhookConfig['action_url'];

                    if ($webhookType === Leads_Model_LeadsWebhookNotificationsModel::WEBHOOK_TYPE_LEAD) {
                        self::processLeadNotification($notificationUrl, $notificationData);
                    }

                    if ($webhookType === Leads_Model_LeadsWebhookNotificationsModel::WEBHOOK_TYPE_ORDER) {
                        self::processOrderNotification($notificationUrl, $notificationData);
                    }
                }
            }
        }
    }

    /**
     * Process lead notificatiton
     *
     * @param string $notificationUrl notification endpoint url
     * @param array $notificationsData notification data
     */
    public static function processLeadNotification($notificationUrl, $notificationsData)
    {
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $leadsWebhookNotificationsMapper = Leads_Mapper_LeadsWebhookNotificationsMapper::getInstance();

        foreach ($notificationsData as $notificationId => $notificationData) {
            $where = $leadMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $notificationData['resource_id']);
            $leadsData = $leadMapper->fetchAll(
                $where,
                null,
                null,
                null,
                true,
                true,
                false,
                '',
                false,
                true,
                false,
                true,
                true,
                true
            );

            if (!empty($leadsData)) {
                if (!empty($leadsData['tagIds'])) {
                    $tags = $leadsTagsMapper->getTagsForLeadApi($leadsData['id']);
                    if (!empty($tags)) {
                        $leadsData['tags'] = $tags;
                    } else {
                        $leadsData['tags'] = array();
                    }
                } else {
                    $leadsData['tags'] = array();
                }

                self::notifyCrmInfo($notificationUrl, $leadsData);

                $leadsWebhookNotificationsModel = $leadsWebhookNotificationsMapper->find($notificationId);
                if ($leadsWebhookNotificationsModel instanceof Leads_Model_LeadsWebhookNotificationsModel) {
                    $leadsWebhookNotificationsModel->setStatus(Leads_Model_LeadsWebhookNotificationsModel::STATUS_SENT);
                    $leadsWebhookNotificationsModel->setSentAt(Tools_System_Tools::convertDateFromTimezone('now'));
                    $leadsWebhookNotificationsModel->setSentNotificationsData(json_encode($leadsData));
                    $leadsWebhookNotificationsMapper->save($leadsWebhookNotificationsModel);
                }
            } else {
                $leadsWebhookNotificationsModel = $leadsWebhookNotificationsMapper->find($notificationId);
                if ($leadsWebhookNotificationsModel instanceof Leads_Model_LeadsWebhookNotificationsModel) {
                    $leadsWebhookNotificationsMapper->delete($notificationId);
                }
            }
        }
    }

    /**
     * Process order notificatiton
     *
     * @param string $notificationUrl notification endpoint url
     * @param array $notificationsData notification data
     */
    public static function processOrderNotification($notificationUrl, $notificationsData)
    {
        $cartCartSessionMapper = Models_Mapper_CartSessionMapper::getInstance();
        $leadsWebhookNotificationsMapper = Leads_Mapper_LeadsWebhookNotificationsMapper::getInstance();

        foreach ($notificationsData as $notificationId => $notificationData) {
            $orderId = $notificationData['resource_id'];
            $cartSessionModel = $cartCartSessionMapper->find($orderId);
            $leadsWebhookNotificationsModel = $leadsWebhookNotificationsMapper->find($notificationId);
            if (!$leadsWebhookNotificationsModel instanceof Leads_Model_LeadsWebhookNotificationsModel) {
                $leadsWebhookNotificationsMapper->delete($notificationId);
                continue;
            }

            if ($cartSessionModel instanceof Models_Model_CartSession) {
                $webHookActionType = $notificationData['webhook_action_type'];
                if ($webHookActionType === Leads_Model_LeadsWebhookNotificationsModel::WEBHOOK_ACTION_TYPE_ORDER_PARTIAL) {
                    $orderData = json_decode($leadsWebhookNotificationsModel->getAdditionalInfo(), true);
                } else {
                    $orderData = $cartSessionModel->toArray();

                    foreach ($orderData['cartContent'] as $key => $cartItem) {

                    }
                }

                if (!empty($orderData)) {
                    self::notifyCrmInfo($notificationUrl, $orderData);
                    $leadsWebhookNotificationsModel->setStatus(Leads_Model_LeadsWebhookNotificationsModel::STATUS_SENT);
                    $leadsWebhookNotificationsModel->setSentAt(Tools_System_Tools::convertDateFromTimezone('now'));
                    $leadsWebhookNotificationsModel->setSentNotificationsData(json_encode($orderData));
                    $leadsWebhookNotificationsMapper->save($leadsWebhookNotificationsModel);

                }
            } else {
                $leadsWebhookNotificationsMapper->delete($notificationId);
            }
        }
    }

    /**
     * Send request on action from config
     *
     * @param string $actionUrl action url
     * @param array $data data
     * @return false|mixed
     */
    public static function notifyCrmInfo($actionUrl, $data)
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $actionUrl);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/json"));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_TIMEOUT, 60);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
        $response = curl_exec($curl);
        curl_close($curl);
        return json_decode($response, true);
    }

}
