<?php

class Widgets_Leademailreports_Leademailreports extends Widgets_Abstract
{

    /**
     * Website helper
     *
     * @var null
     */
    protected $_websiteHelper = null;

    /**
     * @var bool
     */
    protected $_cacheable = false;

    /**
     * Session helper
     *
     * @var bool
     */
    protected $_sessionHelper = false;

    /**
     * @var null
     */
    protected $_layout = null;

    /**
     * User report config data
     *
     * @var array
     */
    protected $_userReportConfigData = array();

    protected $_userModel = null;

    /**
     * Registry key
     *
     * @var string
     */
    private static $_zendRegistryKey = 'leadEmailReportsUserObserver';

    protected function _init()
    {
        parent::_init();
        $this->_view = new Zend_View(array(
            'scriptPath' => __DIR__ . '/views/'
        ));


        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        $this->_websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $this->_view->websiteUrl = $this->_websiteHelper->getUrl();
        $this->_websiteUrl = $this->_websiteHelper->getUrl();
        $this->_translator = Zend_Registry::get('Zend_Translate');

        $this->_view->setHelperPath(APPLICATION_PATH . '/views/helpers/');
        $this->_view->addHelperPath('ZendX/JQuery/View/Helper/', 'ZendX_JQuery_View_Helper');
        $this->_view->addScriptPath($this->_websiteHelper->getPath() . 'seotoaster_core/application/views/scripts/');
        // init helpers
        $this->_configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    protected function _load()
    {
        if (empty($this->_options[0])) {
            return $this->_translator->translate(Leads::TRANSLATION_PREFIX . 'Option not specified');
        }

        if (!empty($this->_toasterOptions['observableModel']) && is_object($this->_toasterOptions['observableModel'])) {
            $observableModel = $this->_toasterOptions['observableModel'];
            if ($observableModel instanceof Application_Model_Models_User) {
                $regKey = self::$_zendRegistryKey . '_' . $observableModel->getId();
                if (Zend_Registry::isRegistered($regKey)) {
                    $this->_userReportConfigData = Zend_Registry::get($regKey);
                } else {
                    $this->_userReportConfigData = array();

                    if (!empty($this->_userReportConfigData)) {
                        Zend_Registry::set($regKey, $this->_userReportConfigData);
                    }
                }

                $this->_userModel = $observableModel;

            } else {
                $this->_userModel = null;
            }
        }

        $methodName = '_render' . ucfirst($this->_options[0]);
        if (method_exists($this, $methodName)) {
            return $this->$methodName();

        }

        return $this->_translator->translate(Leads::TRANSLATION_PREFIX . 'Method doesn\'t exists');
    }

    protected function _renderTasks()
    {

        if (!$this->_userModel instanceof Application_Model_Models_User) {
            return '';
        }

        $userId = $this->_userModel->getId();
        $userTimezone = $this->_userModel->getTimezone();
        $subtype = Tools_LeadEmailReportsTools::TASKS_COMPLETED;
        if (!empty($this->_options['1'])) {
            $subtype = $this->_options['1'];
        }

        if (!in_array($subtype, Tools_LeadEmailReportsTools::$_allowedTasksSubtypes)) {
            return '';
        }

        $receiveReportsTypesList = $this->_userModel->getReceiveReportsTypesList();
        if (empty($receiveReportsTypesList)) {
            return '';
        }

        $receiveReportsTypes = explode(',', $receiveReportsTypesList);

        if (!in_array(Tools_LeadEmailReportsTools::REPORT_TYPE_TASKS, $receiveReportsTypes)) {
            return '';
        }

        $tasksData = Tools_LeadEmailReportsTools::getTasksReportsData($userId, $userTimezone, $subtype);
        $userTimezone = $this->_userModel->getTimezone();

        $this->_view->tasksData = $tasksData;
        $this->_view->subtype = $subtype;
        $this->_view->userTimezone = $userTimezone;
        return $this->_view->render('lead-email-tasks-reports.phtml');
    }

    protected function _renderMeetings()
    {
        if (!$this->_userModel instanceof Application_Model_Models_User) {
            return '';
        }

        $userId = $this->_userModel->getId();
        $userTimezone = $this->_userModel->getTimezone();
        $receiveReportsTypesList = $this->_userModel->getReceiveReportsTypesList();
        if (empty($receiveReportsTypesList)) {
            return '';
        }

        $subtype = Tools_LeadEmailReportsTools::TASKS_COMPLETED;
        if (!empty($this->_options['1'])) {
            $subtype = $this->_options['1'];
        }

        if (!in_array($subtype, Tools_LeadEmailReportsTools::$_allowedMeetingSubtypes)) {
            return '';
        }

        if (empty($userTimezone)) {
            $userTimezone = 'UTC';
        }

        $receiveReportsTypes = explode(',', $receiveReportsTypesList);

        if (!in_array(Tools_LeadEmailReportsTools::REPORT_TYPE_MEETINGS, $receiveReportsTypes)) {
            return '';
        }

        $meetingsData = Tools_LeadEmailReportsTools::getMeetingReportsData($userId, $userTimezone, $subtype);

        $this->_view->meetingsData = $meetingsData;
        $this->_view->subtype = $subtype;
        $this->_view->userTimezone = $userTimezone;
        return $this->_view->render('lead-email-meetings-reports.phtml');

    }

    protected function _renderNotes()
    {
        if (!$this->_userModel instanceof Application_Model_Models_User) {
            return '';
        }

        $userId = $this->_userModel->getId();
        $userTimezone = $this->_userModel->getTimezone();
        if (empty($userTimezone)) {
            $userTimezone = 'UTC';
        }

        $receiveReportsTypesList = $this->_userModel->getReceiveReportsTypesList();
        if (empty($receiveReportsTypesList)) {
            return '';
        }

        $receiveReportsTypes = explode(',', $receiveReportsTypesList);

        if (!in_array(Tools_LeadEmailReportsTools::REPORT_TYPE_NOTES, $receiveReportsTypes)) {
            return '';
        }

        $notesData = Tools_LeadEmailReportsTools::getNotesReportsData($userId, $userTimezone);

        $this->_view->notesData = $notesData;
        return $this->_view->render('lead-email-notes-reports.phtml');
    }


    protected function _renderUserid()
    {
        if (!$this->_userModel instanceof Application_Model_Models_User) {
            return '';
        }

        return $this->_userModel->getId();
    }

    protected function _renderFullname()
    {
        if (!$this->_userModel instanceof Application_Model_Models_User) {
            return '';
        }

        return $this->_userModel->getFullName();
    }

    protected function _renderReportdate()
    {
        if (!$this->_userModel instanceof Application_Model_Models_User) {
            return '';
        }

        $userTimezone = $this->_userModel->getTimezone();
        if (empty($userTimezone)) {
            $userTimezone = 'UTC';
        }

        $date = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $userTimezone, 'd-M-Y');

        return $date;
    }

    protected function _renderReportdateyesterday()
    {
        if (!$this->_userModel instanceof Application_Model_Models_User) {
            return '';
        }

        $userTimezone = $this->_userModel->getTimezone();
        if (empty($userTimezone)) {
            $userTimezone = 'UTC';
        }

        $date = Tools_System_Tools::convertDateFromTimezone('-1 day', 'UTC', $userTimezone, 'd-M-Y');


        $excludeWeekends = $this->_userModel->getExcludeWeekends();
        $currentDayOfTheWeekNumber = (int)Tools_System_Tools::convertDateFromTimezone('now', 'UTC', $userTimezone, 'N');
        if (!empty($excludeWeekends)) {
            if ($currentDayOfTheWeekNumber === 1) {
                $date = Tools_System_Tools::convertDateFromTimezone('-3 day', 'UTC', $userTimezone, 'd-M-Y');
            }
        }

        return $date;
    }

}
