<?php

class Widgets_Leadquote_Leadquote extends Widgets_Abstract
{

    /**
     * Quote lead custom params key
     *
     * @var string
     */
    private static $_registryLeadQuoteCustomParamsKey = 'leadQuoteCustomParamsKey';

    /**
     * Quote object registry key
     *
     * @var string
     */
    private static $_registryQuoteObject = 'leadQuoteRegistryKey';

    /**
     * Quote lead model registry key
     *
     * @var string
     */
    private static $_registryLeadModel = 'leadQuoteLeadModelKey';

    /**
     * Website helper
     *
     * @var null
     */
    protected $_websiteHelper = null;

    /**
     * @var bool
     */
    protected $_cacheable = false;

    /**
     * Session helper
     *
     * @var bool
     */
    protected $_sessionHelper = false;

    /**
     * lead custom params data
     *
     * @var bool
     */
    protected $_leadCustomParamsData = array();

    /**
     * @var Quote_Models_Model_Quote|null
     */
    protected $_quoteModel = null;

    /**
     * @var Leads_Model_LeadsModel|null
     */
    protected $_leadModel = null;

    /**
     * @var null
     */
    protected $_layout = null;


    protected function _init()
    {
        parent::_init();
        $this->_view = new Zend_View(array(
            'scriptPath' => __DIR__ . '/views/'
        ));


        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        $this->_websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $this->_view->websiteUrl = $this->_websiteHelper->getUrl();
        $this->_websiteUrl = $this->_websiteHelper->getUrl();
        $this->_translator = Zend_Registry::get('Zend_Translate');

        $this->_view->setHelperPath(APPLICATION_PATH . '/views/helpers/');
        $this->_view->addHelperPath('ZendX/JQuery/View/Helper/', 'ZendX_JQuery_View_Helper');
        $this->_view->addScriptPath($this->_websiteHelper->getPath() . 'seotoaster_core/application/views/scripts/');
        // init helpers
        $this->_configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    protected function _load()
    {
        if (empty($this->_options[0])) {
            return $this->_translator->translate(Leads::TRANSLATION_PREFIX . 'Option not specified');
        }

        if (empty($this->_options[1])) {
            return $this->_translator->translate(Leads::TRANSLATION_PREFIX . 'Empty quote id');
        }

        $quoteId = $this->_options[1];

        $quoteRegistryKey = self::$_registryQuoteObject . '_' . $quoteId;

        if (Zend_Registry::isRegistered($quoteRegistryKey)) {
            $this->_quoteModel = Zend_Registry::get($quoteRegistryKey);
        } else {
            $quoteModel = Quote_Models_Mapper_QuoteMapper::getInstance()->find($quoteId);
            if ($quoteModel instanceof Quote_Models_Model_Quote) {
                Zend_Registry::set($quoteRegistryKey, $quoteModel);
                $this->_quoteModel = $quoteModel;
            } else {
                $this->_quoteModel = null;
            }
        }

        if (!$this->_quoteModel instanceof Quote_Models_Model_Quote) {
            return '';
        }

        $userId = $this->_quoteModel->getUserId();

        if (empty($userId)) {
            return '';
        }

        $registryLeadKey = self::$_registryLeadModel . '_' . $userId;
        if (Zend_Registry::isRegistered($registryLeadKey)) {
            $this->_leadModel = Zend_Registry::get($registryLeadKey);
        } else {
            $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
            if ($userModel instanceof Application_Model_Models_User) {
                $userEmail = $userModel->getEmail();
                $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                $leadModel = $leadMapper->findByEmail($userEmail);
                if ($leadModel instanceof Leads_Model_LeadsModel) {
                    $this->_leadModel = $leadModel;
                } else {
                    $this->_leadModel = null;
                }
            } else {
                $this->_leadModel = null;
            }
        }

        if (!$this->_leadModel instanceof Leads_Model_LeadsModel) {
            return '';
        }

        $this->_initLeadCustomParams($this->_leadModel->getId());

        $methodName = '_render' . ucfirst($this->_options[0]);
        if (method_exists($this, $methodName)) {
            return $this->$methodName();
        }

        return $this->_translator->translate(Leads::TRANSLATION_PREFIX . 'Method doesn\'t exists');
    }

    protected function _renderCustomParam()
    {
        if (!empty($this->_leadCustomParamsData) && !empty($this->_options['2'])) {
            if (array_key_exists($this->_options['2'], $this->_leadCustomParamsData)) {
                $fieldInfo = $this->_leadCustomParamsData[$this->_options['2']];
                if ($fieldInfo['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_DATE && !empty($fieldInfo['param_value'])) {
                    $defaultTimezone = 'UTC';
                    $formDate = Tools_System_Tools::convertDateFromTimezone($fieldInfo['param_value'], $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d H:i:s');
                    return $formDate;
                }

                return $fieldInfo['param_value'];
            }

            return '';
        }

        return '';

    }

    private function _initLeadCustomParams($leadId)
    {
        $registryKey = self::$_registryLeadQuoteCustomParamsKey . '_' . $leadId;

        if (Zend_Registry::isRegistered($registryKey)) {
            $this->_leadCustomParamsData = Zend_Registry::get($registryKey);
        } else {
            $leadCustomParams = Leads_Mapper_LeadsCustomParamsMapper::getInstance()->getParamsNames($leadId);
            if (empty($leadCustomParams)) {
                Zend_Registry::set($registryKey, array());
                $this->_leadCustomParamsData = array();
            } else {
                $this->_leadCustomParamsData = $leadCustomParams;
            }
        }

    }

}
